/**
 * 对象排序
 *
 * @param property {string}
 * @return {function}
 * */
function compare(property) {
  return function (obj1, obj2) {
    let value1 = obj1.ui[property];
    let value2 = obj2.ui[property];
    return value1 - value2;
  }
}


/**
 * 数组排序
 *
 * @param a {number}
 * @param b {number}
 * @return {number}
 * */
function sortArray(a, b) {
  return a - b;
}


/**
 * 数组去重
 * @param array {array}
 * @return {array}
 * */
function unique(array) {
  const res = new Map();
  return array.filter(d => !res.has(d) && res.set(d, 1))
}


/**
 * 数组中对象去重
 * */
function duplicate(arr, type) {
  let obj = {};
  let newArr = arr.reduce((cur, next) => {
    obj[next[type]] ? "" : obj[next[type]] = true && cur.push(next);
    return cur;
  }, []);
  return newArr;
}


/**
 * 替换emoji表情
 * */
const filterEmoji = (name) => {
  let str = name.replace(/[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF][\u200D|\uFE0F]|[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF]|[0-9|*|#]\uFE0F\u20E3|[0-9|#]\u20E3|[\u203C-\u3299]\uFE0F\u200D|[\u203C-\u3299]\uFE0F|[\u2122-\u2B55]|\u303D|[\A9|\AE]\u3030|\uA9|\uAE|\u3030/ig, " ");
  return str;
};


/**
 * 替换空格键
 * */ 
const filterSpace = (name) => {
  let str = name.replace(/\s+/g, '');
  return str;
};


/**
 * 手势动作处理 旋转
 * */
const handleRotateData = (curAngle, perAngle) => {
  let temp = 0;
  if (Math.abs(curAngle) > Math.abs(perAngle)) {
    if (curAngle === 90 || curAngle === -90) {
      temp = -curAngle + Math.abs(perAngle); // 逆时针
    } else if (curAngle > 0 && perAngle < 0) {
      temp = curAngle + perAngle; // 逆时针
    } else if (curAngle < 0 && perAngle > 0 && (Math.abs(curAngle) > perAngle)) {
      temp = curAngle + perAngle; // 逆时针
    } else {
      temp = curAngle - perAngle; // 逆时针
    }
  } else {
    if (perAngle > 0 && curAngle > 0) {
      temp = curAngle - perAngle; // 逆时针
    } else {
      if (perAngle === 90) {
        temp = -curAngle + Math.abs(perAngle); // 逆时针
      } else if (perAngle === -90) {
        temp = Math.abs(curAngle) - Math.abs(perAngle); // 逆时针
      } else if (perAngle < 0 && curAngle > 0 && Math.abs(perAngle) > Math.abs(curAngle)) {
        temp = perAngle + perAngle; // 逆时针
      } else {
        temp = curAngle + Math.abs(perAngle); // 顺时针
      }
    }
  }
  return temp;
};

export {
  compare,
  sortArray,
  unique,
  duplicate,
  filterEmoji,
  filterSpace,
  handleRotateData,
}
