import Affine from './affine.js';
/**
 * 引入dialog
 * */
let app = getApp();


/**
 * this
 * */
let editPage;


/**
 * sdk 私有变量对象
 * */
let privateVariableJson = {};


/**
 * 初始化编辑所需参数
 *
 * @returns
 * {
 *    windowWidth,          屏幕宽度
 *    windowHeight,         屏幕高度
 *    canvasW,              画布宽度
 *    canvasH,              画布高度
 *    showCanvas,           canvas显示和隐藏
 *    globalCanvasSizeArray 所有canvas宽高的数组
 * }
 * */
const setting = () => {
  editPage.setData({
    windowWidth: 0,
    windowHeight: 0,
    canvasW: 0,
    canvasH: 0,
    showCanvas: 0,
    globalCanvasSizeArray: [],
  }, () => {
    privateVariableJson.windowWidth = 0;
    privateVariableJson.windowHeight = 0;
    privateVariableJson.canvasW = 0;
    privateVariableJson.canvasH = 0;
    privateVariableJson.showCanvas = 0;
    privateVariableJson.globalCanvasSizeArray = [];
  })
};


/**
 * 解析模板数据
 *
 * @param configJson {object}
 * @returns {assetList, groupList}
 * */
const parseData = (configJson) => {
  let _this = editPage;
  _this.setData({
    'assetList': configJson.uiAssets,
    'groupList': configJson.configJson.ui_group,
  }, function () {
    const {
      assetList,
      groupList
    } = _this.data;
    privateVariableJson.assetList = assetList;
    privateVariableJson.groupList = groupList;
    // parseConfigJson(privateVariableJson.assetList);
  })
};


/**
 * 解析模板换行符
 * @param configJson {object}
 * */
const parseNewLine = (configJson) => {
  let temp = {
    ...configJson
  };
  // temp.assets.map(x => {
  //   if (x.ui && x.ui.type !== 1) {
  //     x.ui.default = x.ui.default.replace(/↵/g, '\n');
  //   }
  // });

  parseData(temp);
};



/**
 * 初始化素材(图片和文字) 矩阵
 *
 * @params {
 *     json {array}
 *     canvasW {number} 画布宽度
 *     canvasH {number} 画布高度
 * }
 * @returns {picArray,textareaArray}
 * */
const initMatrix = (json, GroupSize, windowHeight, windowWidth) => {
  let _this = editPage;
  let jsonArray = [];

  // 给每个设原始矩阵
  for (let widgetAll in json) {
    let all = json[widgetAll].all;
    let picArray = []; // 图片数组
    let textareaArray = []; // 文字数组
    let obj = {};
    let scale = 0;
    let scaleBottom = 0;
    let scaleTop = windowHeight / GroupSize[widgetAll].size[1],
      width = GroupSize[widgetAll].size[0] * scaleTop;

    let scaleHeight = windowWidth / GroupSize[widgetAll].size[0],
      height = GroupSize[widgetAll].size[1] * scaleHeight,
      maxScale = height / GroupSize[widgetAll].size[1],
      maxWidth = GroupSize[widgetAll].size[0] * maxScale;

    // let globalWidthScale = app.all.parseConfig.getWidthScaleByGroup(windowHeight, GroupSize[widgetAll].groupKey);
    // if (globalWidthScale.width > windowWidth) {
    //   let globalHeightScale = app.all.parseConfig.getHeightScaleByGroup(windowWidth, GroupSize[widgetAll].groupKey),
    //     globalWidthScale1 = app.all.parseConfig.getWidthScaleByGroup(globalHeightScale.height, GroupSize[widgetAll].groupKey);
    //   console.log(globalHeightScale, globalWidthScale1, windowWidth);
    // }

    let scaleBottomS = 112 / windowHeight,
      widthBootom = width * scaleBottomS;

    //宽度超出定高
    let scaleBottomHeight = 112 / height,
      maxWidthBottom = maxWidth * scaleBottomHeight;
    if (width <= windowWidth) {
      scale = scaleTop;
      scaleBottom = scaleBottomS;
      obj.widthBox = width;
      obj.heightBox = windowHeight;
      obj.widthBootom = widthBootom;
      obj.scaleBox = scale;
    } else {
      scale = scaleHeight;
      scaleBottom = scaleBottomHeight;
      obj.widthBox = maxWidth;
      obj.heightBox = height;
      obj.widthBootom = maxWidthBottom;
      obj.scaleBox = scaleHeight;
    }
    // let scaleBottom = scaleBottomHeight;
    obj.HeightBootom = 112;
    for (let widget of all) {
      if (widget.asset.ui) {
        let ui = widget.asset.ui;
        let initOrigin = new Affine();
        // 原始矩阵
        let anchor = {
          x: ui.a[0],
          y: ui.a[1]
        };
        let position = {
          x: ui.p[0],
          y: ui.p[1]
        };
        let zoom = {
          x: ui.s[1],
          y: ui.s[1]
        };
        let rotation = ui.r;
        initOrigin.set(anchor, position, zoom, rotation);
        widget.matrix = initOrigin;
        // widget.inverse = initOrigin.getInverse(); // 记录原始矩阵的逆矩阵

        // let usw = canvasW / groupList[ui.group - 1].size[0];
        // tempScale = usw;
        // let ush = canvasH / groupList[ui.group - 1].size[1];

        // widget.canvasLeftOffset = (windowWidth - canvasW) / 2;
        // widget.canvasTopOffset = 20;

        let preScale = new Affine();
        let anchorScale = {
          x: 0,
          y: 0
        };
        let positionScale = {
          x: 0,
          y: 0
        };
        let zoomScale = {
          x: scale,
          y: scale
        };
        preScale.set(anchorScale, positionScale, zoomScale, 0);
        widget.matrix.multiply(preScale);

        let newMatrix = new Affine();
        // 原始矩阵
        let anchorImg = {
          x: 0,
          y: 0
        };
        let positionImg = {
          x: 0,
          y: 0
        };
        let zoomImg = {
          x: 1,
          y: 1
        };
        let rotationImg = 0;
        newMatrix.set(anchorImg, positionImg, zoomImg, rotationImg);
        widget.newMatrix = newMatrix;

        if (ui.type === 1) {
          widget.assetName =  app.all.assetList[widget.assetName];
          widget.scaleBottom = scaleBottom;
          widget.imgSize = [widget.originSize[0], widget.originSize[1]];
          widget.videoSize = [widget.originSize[0], widget.originSize[1]];
          widget.scaleImg = 1;
          widget.pageX = 0;
          widget.pageY = 0;
          widget.rotateImg = 0;
          widget.imgUrl = "";
          picArray.push(widget);
        } else {
          textareaArray.push(widget);
        }
      }
    }
    obj.picArray = picArray
    obj.textareaArray = textareaArray
    obj.bg = json[widgetAll].bg
    obj.fg = json[widgetAll].fg
    obj.groupKey = json[widgetAll]["groupKey"];
    jsonArray.push(obj)
  }
  _this.setData({
    currentGroup: jsonArray[0],
  })
  _this.setData({
    groups: jsonArray,
  })
};





export default {


  /**
   * 初始化编辑器的宽高
   *
   * @param json Object
   * @param sizeConfig {object}
   * @param page this指针
   * */
  initEditor: function (json, sizeConfig, page, all) {
    let GroupSize = json.configJson.ui_group
    let groups = json.groups;
    // 校验是否是type = 1, 然后是否是2.0 ,然后模板里面是否有数据
    if (json.uiAssets.length === 0) {
      app.showWord('请检查模板是否正确!');
      return false;
    }

    // deep = 1; // 初始化遍历层数
    // recordStatus = 1; // 初始化记录值
    editPage = page;
    setting();
    parseNewLine(json);
    let _this = page;
    let {
      windowWidth: vww,
      windowHeight: vwh,
      editorWidth: vcw,
      editorHeight: vch
    } = sizeConfig;
    wx.getSystemInfo({
      success: function (res) {
        //屏幕占比
        _this.setData({
          // 限制用户传入的宽度高度过大
          windowWidth: !!vww ? vww > res.windowWidth ? res.windowWidth : vww : res.windowWidth,
          windowHeight: !!vwh ? vwh > res.windowHeight ? res.windowHeight : vwh : res.windowHeight,
          canvasH: 320,
        }, () => {
          const {
            canvasH,
            windowWidth,
            windowHeight
          } = _this.data;
          privateVariableJson.canvasH = canvasH;
          privateVariableJson.windowWidth = windowWidth;
          privateVariableJson.windowHeight = windowHeight;
          initMatrix(all, GroupSize, privateVariableJson.windowHeight - 300, privateVariableJson.windowWidth - 20);
        })
      },
    })
  },
}