import React, { PureComponent } from "react";
import { Modal, Button, Slider } from "antd";
import styles from "./index.less";
import Moveable from "components/EditMain/Moveable";
import { getImageInfo } from "utils";
import Affine from "utils/lib/Affine";
import { history } from "umi";
import MatrixUtil from "utils/lib/MatrixUtil";
import app from '@/utils/lib/app'
import { connect } from "react-redux";
class ClipImage extends PureComponent {
  constructor(props) {
    super(props);
     this.productId = app.getUrlValue('templateId');

    this.loading = false;
    this.containerHeight = 350; // 最大高度
    this.containerWidth = 450; // 最大宽度
    this.containerAvailableHeight = 450; // 左侧容器的高度
    this.modalRightWidth = 300; // 右侧容器的宽度
    this.modalWidth = 842; // modal 的宽度

    this.initImageClip();
  }

  initImageClip = () => {
    const productId = this.productId;
    this.assetName = this.props.material.assetName;
    this.editSize = this.props.material.originSize;

    const { batchClipImageGlobalVar } = this.props;

    this.editOriginScale = this.editSize[0] / this.editSize[1];
    this.editHeightScale = this.containerHeight / this.editSize[1];
    this.editWidthScale = this.containerWidth / this.editSize[0];
    this.editScale =
      this.editHeightScale > this.editWidthScale
        ? this.editWidthScale
        : this.editHeightScale;

    this.containerSize = [
      this.editSize[0] * this.editScale,
      this.editSize[1] * this.editScale,
    ];

    this.modalLeftWidth = this.modalWidth - this.modalRightWidth;
    this.modalLeftOffset = (this.modalLeftWidth - this.containerSize[0]) / 2; // 裁剪框距离左侧距离
    this.modalTopOffset =
      (this.containerAvailableHeight - this.editScale * this.editSize[1]) / 2; // 裁剪框距离顶部距离

    this.containerVar = {
      position: {
        x:
          (document.body.clientWidth - this.modalWidth) / 2 +
          this.modalLeftOffset,
        y: 155 + this.modalTopOffset,
      },
      anchor: {
        x: 0,
        y: 0,
      },
      scale: {
        x: this.editScale,
        y: this.editScale,
      },
      rotate: 0,
    };

    this.containerLeftOffset =
      -this.containerVar.position.x + this.modalLeftOffset;
    this.containerTopOffset =
      -this.containerVar.position.y + this.modalTopOffset;

    this.frame = {
      referPoint: {
        x: 0,
        y: 0,
      },
      referScale: {
        x: 1,
        y: 1,
      },
      referRotate: 0,
    };

    let {
      cut_origin_draft: cutOriginDraft,
      material,
      originAssetList,
    } = this.props;

    this.currentImageUrl = cutOriginDraft[material["assetName"]]
      ? cutOriginDraft[material["assetName"]]
      : originAssetList[material["assetName"]];

    // 使用上一次的矩阵信息
    // 如果图片被替换过，imageUrl 不相等, 然后执行 自动适配的逻辑
    // console.log("++++ use global data", batchClipImageGlobalVar[productId]);
    if (
      batchClipImageGlobalVar[productId] &&
      batchClipImageGlobalVar[productId][this.assetName] &&
      batchClipImageGlobalVar[productId][this.assetName].imageUrl ==
        this.currentImageUrl
    ) {
      let {
        anchor,
        position,
        scale,
        rotate,
        imageSize,
        imageUrl,
      } = batchClipImageGlobalVar[productId][this.assetName];

      this.state = {
        stateMatAnchor: anchor,
        stateMatPosition: position,
        stateMatScale: scale,
        stateMatRotate: rotate,
        imageSize: imageSize,
        imageUrl: imageUrl,
        imageList: [],
        imageDomRef: null,
        stateShow: "none",
      };
    } else {
      this.state = {
        stateMatAnchor: { x: 1, y: 1 },
        stateMatPosition: { x: 0, y: 0 },
        stateMatScale: { x: 1, y: 1 },
        stateMatRotate: 0,
        imageSize: [1, 1],
        imageDomRef: null,
        stateShow: "none",
      };

      // 获取图片原始尺寸后，自动适配
      getImageInfo(this.currentImageUrl).then(({ width, height }) => {
        this.imageSize = [width, height];

        let adapt = MatrixUtil.autoAdapt(this.imageSize, this.editSize);

        this.setState({
          stateMatAnchor: MatrixUtil.toVec2(adapt.anchor),
          stateMatPosition: MatrixUtil.toVec2(adapt.position),
          stateMatScale: MatrixUtil.toVec2(adapt.scale),
          stateMatRotate: adapt.rotate,
          imageSize: this.imageSize,
          stateShow: "block",
        });
      });
    }
  };

  getWidthScale(height, size) {
    let scale = height / size[1],
      width = size[0] * scale;
    return {
      scale: scale,
      width: width,
    };
  }

  setImageSize(imageUrl) {
    getImageInfo(imageUrl).then(({ width, height }) => {
      const { imageSize } = this.state;
      if (imageSize[0] !== width || imageSize[1] !== height) {
        this.setState({
          imageSize: [width, height],
        });
      }
    });
  }

  createCanvas = (size) => {
    const canvas = document.createElement("canvas"),
      context = canvas.getContext("2d");
    canvas.width = size[0];
    canvas.height = size[1];

    return {
      canvas: canvas,
      context: context,
    };
  };

  hideModal = () => {
    const { setClipImageModelStatus } = this.props;
    this.setState({
      stateShow: "none",
    });
    setClipImageModelStatus(false);
  };

  getLocalPos = (e) => {
    return {
      x: e.clientX,
      y: e.clientY,
    };
  };

  mousePressEvent = (e) => {
    let { stateMatRotate, stateMatScale } = this.state;
    this.frame.referRotate = stateMatRotate;
    this.frame.referScale = {
      x: stateMatScale.x,
      y: stateMatScale.y,
    };
    this.frame.referPoint = MatrixUtil.getTransformMatrix(this.containerVar)
      .getInverse()
      .transform({
        x: this.getLocalPos(e).x,
        y: this.getLocalPos(e).y,
      });

    // console.log("mousePressEvent", this.frame);
  };

  handleScaleStart = (e) => {
    this.mousePressEvent(e);
  };

  handleScale = (e) => {
    let { direction, clientX, clientY } = e;
    const {
      imageSize: imgSize,
      stateMatAnchor,
      stateMatPosition,
      stateMatScale,
      stateMatRotate,
    } = this.state;
    let { containerVar } = this;

    clientX = this.getLocalPos(e).x;
    clientY = this.getLocalPos(e).y;

    let scaleInfo = MatrixUtil.getScaleInfo(direction, imgSize);
    let { anchor } = scaleInfo;

    let local = MatrixUtil.getTransformMatrix(containerVar)
      .getInverse()
      .transform({ x: clientX, y: clientY });
    let position = MatrixUtil.composeTransformMatrix(
      stateMatAnchor,
      stateMatPosition,
      stateMatScale,
      stateMatRotate,
    ).getPositionFromAnchor(anchor.x, anchor.y);

    let { scale } = MatrixUtil.calculateScale(
      scaleInfo,
      local,
      position,
      stateMatRotate,
      this.frame.referScale,
    );

    // 初始化一个目标矩阵，锚点和位置都在右下角
    var newAffine = new Affine();
    newAffine.set(anchor, position, scale, stateMatRotate);

    position = newAffine.getPositionFromAnchor(
      stateMatAnchor.x,
      stateMatAnchor.y,
    );

    this.setState({
      stateMatPosition: position,
      stateMatScale: scale,
    });
  };

  handleScaleEnd = ({ lastEvent }) => {};

  handleRotateStart = (e) => {
    this.mousePressEvent(e);
  };

  handleRotate = (e) => {
    let { clientX, clientY } = e;
    let {
      imageSize: imgSize,
      stateMatAnchor,
      stateMatPosition,
      stateMatScale,
      stateMatRotate,
    } = this.state;

    let local = MatrixUtil.getTransformMatrix(this.containerVar)
      .getInverse()
      .transform({ x: clientX, y: clientY });
    let position = stateMatPosition;
    let start = {
      x: this.frame.referPoint.x - position.x,
      y: this.frame.referPoint.y - position.y,
    };

    let end = {
      x: local.x - position.x,
      y: local.y - position.y,
    };

    let rotate = this.frame.referRotate + Affine.angleBetween(start, end);
    // console.log("onroate", rotate, stateMatAnchor, start, end);
    this.setState({
      stateMatRotate: rotate,
    });
  };

  handleRotateEnd = (e) => {};

  handleDragStart = (e) => {
    this.mousePressEvent(e);
  };

  handleDrag = (e) => {
    let { clientX, clientY } = e;
    let { stateMatPosition } = this.state;

    let local = MatrixUtil.getTransformMatrix(this.containerVar)
      .getInverse()
      .transform({ x: clientX, y: clientY });
    let position = stateMatPosition;

    this.setState({
      stateMatPosition: {
        x: position.x + local.x - this.frame.referPoint.x,
        y: position.y + local.y - this.frame.referPoint.y,
      },
    });

    this.frame.referPoint = local;
    return false;
  };

  handleDragEnd = ({ lastEvent }) => {};

  dispatchClipInfo = () => {
    const { dispatch } = this.props;
    const productId = this.productId;
    let { batchClipImageGlobalVar } = this.props;

    const {
      stateMatPosition,
      stateMatAnchor,
      stateMatScale,
      stateMatRotate,
      imageSize,
    } = this.state;

    if (!batchClipImageGlobalVar[productId]) {
      batchClipImageGlobalVar[productId] = {};
    }

    batchClipImageGlobalVar[productId][this.assetName] = {
      position: stateMatPosition,
      anchor: stateMatAnchor,
      scale: stateMatScale,
      rotate: stateMatRotate,
      imageSize: imageSize,
      imageUrl: this.currentImageUrl,
    };

    dispatch({
      type: "workBench/save",
      payload: {
        batchClipImageGlobalVar: batchClipImageGlobalVar,
      },
    });
  };

  handleOkButtonClick = (material, materialScale) => {
    this.hideModal();
    const {
        stateMatPosition,
        stateMatAnchor,
        stateMatScale,
        stateMatRotate,
        imageDomRefOrigin,
        imageSize,
      } = this.state,
      { onOk } = this.props,
      { editSize } = this,
      cropCanvas = this.createCanvas(editSize),
      imgWidth = imageSize[0],
      imgHeight = imageSize[1];

    let matrix = new Affine();

    let scale = {
      x: stateMatScale.x, /// this.editScale,
      y: stateMatScale.y, /// this.editScale
    };

    matrix.set(stateMatAnchor, stateMatPosition, scale, stateMatRotate);
    cropCanvas.context.setTransform(
      matrix.mA,
      matrix.mB,
      matrix.mC,
      matrix.mD,
      matrix.mE,
      matrix.mF,
    );


    this.dispatchClipInfo();

    cropCanvas.context.drawImage(imageDomRefOrigin, 0, 0, imgWidth, imgHeight);
    cropCanvas.canvas.toBlob(function (blob) {
      const fileData = new File([blob], "cropImage.png", { type: blob.type });
      onOk && onOk(fileData, true);
    });
  };

  renderTransform = () => {
    const {
        handleDragStart,
        handleDrag,
        handleDragEnd,
        handleScaleStart,
        handleScale,
        handleScaleEnd,
        handleRotateStart,
        handleRotate,
        handleRotateEnd,
      } = this,
      { imageDomRef, imageSize } = this.state;

    return (
      <Moveable
        imageSize={[...imageSize]}
        target={imageDomRef}
        handleScaleStart={handleScaleStart}
        handleScale={handleScale}
        handleScaleEnd={handleScaleEnd}
        handleDragStart={handleDragStart}
        handleDrag={handleDrag}
        handleDragEnd={handleDragEnd}
        handleRotateStart={handleRotateStart}
        handleRotate={handleRotate}
        handleRotateEnd={handleRotateEnd}
      />
    );
  };

  handleScaleSliderChange = (val) => {
    this.setState({
      stateMatScale: {
        x: val,
        y: val,
      },
    });
  };

  handleReSetScaleClick = () => {
    this.setState({
      stateMatScale: {
        x: 1,
        y: 1,
      },
    });
  };

  handleReSetRotateClick = () => {
    this.setState({
      stateMatRotate: 0,
    });
  };

  renderScaleSlider = () => {
    const marks = {
        0: "",
        2.5: "",
        5: "",
        7.5: "",
        10: "",
      },
      { stateMatScale } = this.state;

    return (
      <div className={`flex ${styles.sliderBox}`}>
        <div className={`${styles.sliderLable}`}>缩放</div>
        <div className={`${styles.slider}`}>
          <Slider
            value={Math.abs(stateMatScale.x)}
            marks={marks}
            min={0}
            step={0.1}
            max={10}
            onChange={this.handleScaleSliderChange}
          />
        </div>
        <Button onClick={this.handleReSetScaleClick}>重置</Button>
      </div>
    );
  };

  handleRotateSliderChange = (val) => {
    this.setState({
      stateMatRotate: val,
    });
  };

  renderRotateSlider = () => {
    let marks = {
        0: "",
        72: "",
        144: "",
        216: "",
        288: "",
        360: "",
      },
      { stateMatRotate } = this.state;

    if (stateMatRotate > 360) {
      stateMatRotate %= 360;
    }

    if (stateMatRotate < 0) {
      stateMatRotate += 360;
    }

    return (
      <div className={`flex ${styles.sliderBox}`}>
        <div className={`${styles.sliderLable}`}>旋转</div>
        <div className={`${styles.slider}`}>
          <Slider
            value={Math.abs(stateMatRotate)}
            marks={marks}
            min={0}
            max={360}
            onChange={this.handleRotateSliderChange}
          />
        </div>
        <Button onClick={this.handleReSetRotateClick}>重置</Button>
      </div>
    );
  };
  handleBeforeImage = () => {
    const {
      cut_origin_draft,
      material,
      originAssetList,
      dispatch,
    } = this.props;
    let imageUrl = null;
    if (cut_origin_draft && cut_origin_draft[material["assetName"]]) {
      imageUrl = cut_origin_draft[material["assetName"]];
    } else {
      imageUrl = originAssetList[material["assetName"]];
    }

    dispatch({
      type: "workBench/save",
      payload: {
        cropPicture: imageUrl,
      },
    });
    this.setState({
      imageSize: [1, 1],
      imageDomRef: null,
      stateScaleX: this.originMaterialHeight / this.containerHeight,
      stateScaleY: this.originMaterialHeight / this.containerHeight,
      stateRotate: 0,
      statePositionX: 0,
      statePositionY: 0,
    });
  };

  render() {
    const {
        stateMatPosition,
        stateMatAnchor,
        stateMatScale,
        stateMatRotate,
        stateShow,
        imageSize,
      } = this.state,
      { cropPicture, material, cut_origin_draft } = this.props,
      {
        editSize,
        editScale,
        currentImageUrl,
        containerVar,
        modalWidth,
        modalLeftOffset,
        modalTopOffset,
        containerLeftOffset,
        containerTopOffset,
      } = this;
    //根据图片的尺寸去计算
    // this.setImageSize(currentImageUrl);

    let matrix = MatrixUtil.composeTransformMatrix(
      stateMatAnchor,
      stateMatPosition,
      stateMatScale,
      stateMatRotate,
    );
    let editMatrix = MatrixUtil.getTransformMatrix(this.containerVar);

    return (
      <Modal
        width={`${modalWidth}px`}
        title="裁切图片"
        footer={null}
        visible={true}
        maskClosable={true}
        wrapClassName={styles.antdbody}
        onCancel={this.hideModal}
      >
        <div className={styles.editImageWrap}>
          <div className={`${styles.clipImageContainer}`}>
            <div
              className={`${styles.clipImageLayout}`}
              style={{
                width: editSize[0],
                height: editSize[1],
                transformOrigin: "0px 0px",
                overflow: "hidden",
                left: containerLeftOffset,
                top: containerTopOffset,
                transform: `matrix(${editMatrix.mA}, ${editMatrix.mB}, ${editMatrix.mC}, ${editMatrix.mD}, ${editMatrix.mE}, ${editMatrix.mF})`,
              }}
            >
              <div
                style={{
                  width: imageSize[0],
                  height: imageSize[1],
                  position: "absolute",
                  transformOrigin: "0px 0px",
                  transform: `matrix(${matrix.mA}, ${matrix.mB}, ${matrix.mC}, ${matrix.mD}, ${matrix.mE}, ${matrix.mF})`,
                }}
                ref={(ref) => {
                  this.setState({ imageDomRef: ref });
                  // console.log(ref);
                }}
              >
                <img
                  crossOrigin="anonymous"
                  src={currentImageUrl}
                  alt="cropImage"
                  style={{
                    width: "100%",
                  }}
                  ref={(ref) => {
                    this.setState({ imageDomRefOrigin: ref });
                    // console.log(ref);
                  }}
                />
              </div>
            </div>
            {this.renderTransform()}
            <div
              style={{
                width: editSize[0] * editScale,
                height: editSize[1] * editScale,
                position: "absolute",
                transformOrigin: " 0px 0px",
                // border: "2px dashed #5b4dff",

                left: modalLeftOffset,
                top: modalTopOffset,
              }}
              className={`${styles.cropBorderBox}`}
            />
          </div>

          <div className={styles.flex_right}>
            {this.renderRotateSlider()}
            {this.renderScaleSlider()}
            <div className={`${styles.cropButton}`}>
              <Button
                type="primary"
                onClick={this.handleOkButtonClick.bind(this, material, 1, 1)}
              >
                确定
              </Button>
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

const stateToProps = (state) => {
  const { workBench } = state,
    {
      cut_origin_draft,
      cropPicture,
      originAssetList,
      assetList,
      batchClipImageGlobalVar,
    } = workBench;
  return {
    cut_origin_draft,
    cropPicture,
    assetList,
    originAssetList,
    batchClipImageGlobalVar,
  };
};

export default connect(stateToProps)(ClipImage);
