import React, { PureComponent } from "react";
import styles from "./index.less";
import { renderPreviewAsset } from "utils/lib/renderPreviewAsset";
import MatrixUtil from "utils/lib/MatrixUtil";

class CommonPreview extends PureComponent {
  defaultZIndex = 2;

  renderAccessory = (className, image) => {
    if (!image) return null;
    return (
      <div
        className={`${styles.accessoryImage} ${className}`}
        style={{
          backgroundImage: `url(${image})`,
        }}
      />
    );
  };

  renderImageAsset = (imageAsset, containerScale, imageSrc) => {
    const { currentMaterialKey } = this.props;
    const { defaultZIndex } = this;

    let { p, a, s, r, editSize } = imageAsset.asset.ui;
    let {
      position,
      anchor,
      scale,
      rotate,
      originSize,
      originImageSize,
      transparency,
      assetName: key,
      zIndex,
    } = imageAsset;

    let initMatVar = {
      anchor: a,
      position: p,
      scale: s,
      rotate: r,
    };

    let matVar = {
      anchor: anchor,
      position: position,
      scale: scale,
      rotate: rotate,
    };

    let imageSize = originImageSize ? originImageSize : originSize;

    var matrix = MatrixUtil.getTransformMatrix(matVar).toMatrixString();
    let imageStyle = {
      width: `${imageSize[0]}px`,
      height: `${imageSize[1]}px`,
      position: "absolute",
      opacity: `${transparency}`,
      border: `2px dashed #FF5172`,
      transformOrigin: `0px 0px`,
      transform: matrix,
    };

    let editMatrix = MatrixUtil.getTransformMatrix(initMatVar).toMatrixString();

    let containerStyle = {
      zIndex: zIndex + defaultZIndex,
      overflow: "hidden",
      transformOrigin: "0px 0px",
      width: editSize[0] + "px",
      height: editSize[1] + "px",
      transform: `${editMatrix}`,
    };

    if (currentMaterialKey === key) {
      delete imageStyle.border;
    }

    return (
      <div key={key} className={`${styles.poa}`} style={containerStyle}>
        <img
          className={`${styles.poa}`}
          style={imageStyle}
          src={imageSrc}
          alt="asset"
        />
      </div>
    );
  };

  renderImageMaterial = (
    w,
    h,
    aX,
    aY,
    x,
    y,
    sX,
    sY,
    r,
    t,
    zIndex,
    containerScale,
    src,
    key,
  ) => {
    const { defaultZIndex } = this,
      containerStyle = {
        zIndex: zIndex + defaultZIndex,
        transform: `scale(${containerScale})`,
      },
      imageStyle = {
        left: `${-aX}px`,
        top: `${-aY}px`,
        width: `${w}px`,
        height: `${h}px`,
        opacity: `${t}`,
        transformOrigin: `${aX}px ${aY}px`,
        transform: `translate(${x}px, ${y}px) rotate(${r}deg) scaleX(${sX}) scaleY(${sY})`,
      };

    return (
      <div key={key} className={`${styles.poa}`} style={containerStyle}>
        <img
          className={`${styles.poa}`}
          style={imageStyle}
          src={src}
          alt="asset"
        />
      </div>
    );
  };

  renderTextAsset = (material, containerScale) => {
    const { defaultZIndex } = this,
      {
        anchor,
        position,
        scale,
        rotate,
        fontSetting,
        zIndex,
        transparence,
        originSize,
        assetName,
      } = material,
      {
        defaultText,
        fontSize,
        textAlign,
        textColor,
        maxLength,
        strokeWidth,
        strokeColor,
        font_face,
      } = fontSetting,
      textAligns = {
        0: "left",
        1: "right",
        2: "center",
      },
      containerStyle = {
        zIndex: zIndex + defaultZIndex + 30,
        // transform: `scale(${containerScale})`,
      };

    let textMatrixString = MatrixUtil.composeTransformMatrix(
      anchor,
      position,
      scale,
      rotate,
    ).toMatrixString();
    let textStyle = {
      width: `${originSize[0]}px`,
      height: `${originSize[1]}px`,
      zIndex: zIndex + defaultZIndex + 30,
      opacity: `${transparence}`,
      transformOrigin: `0px 0px`,
      transform: `${textMatrixString}`,
      textAlign: textAligns[textAlign],
      fontSize: `${fontSize}px`,
      color: textColor,
      fontFamily: `${font_face ? font_face.post_script_name : "MyFont"}`,
      WebkitTextStroke: `${material.asset.ui.strokeOverFill
        ? strokeWidth + "px" + strokeColor
        : "none"
        }`,
      
    };
    //设置新的样式
    let newTextStyle = { ...textStyle };

    newTextStyle.zIndex = textStyle.zIndex - 1;
    newTextStyle.color = `${strokeColor}`;
    newTextStyle.fontSize = `${fontSize}px`;
    newTextStyle.WebkitTextStroke = `${strokeWidth}px ${strokeColor}`;
    newTextStyle.lineHeight = `100%`;
    newTextStyle.display =
      !material.asset.ui.strokeOverFill && strokeWidth && strokeColor
        ? "block"
        : "none";

    return (
      <div className={`${styles.poa}`} key={assetName} style={containerStyle}>
        <textarea
          className={styles.textarea}
          style={textStyle}
          readOnly={true}
          maxLength={maxLength}
          value={defaultText}
        />
        {/* 加一层dom进行字体遮挡 */}
        <textarea
          maxLength={maxLength}
          value={defaultText}
          readOnly={true}
          className={styles.newtextarea}
          style={newTextStyle}
        />
      </div>
    );
  };

  renderTextMaterial = (
    w,
    h,
    aX,
    aY,
    x,
    y,
    sX,
    sY,
    r,
    t,
    zIndex,
    containerScale,
    fontSetting,
    key,
    material,
  ) => {
    const { defaultZIndex } = this,
      {
        defaultText,
        fontSize,
        textAlign,
        textColor,
        maxLength,
        strokeWidth,
        strokeColor,
        font_face,
      } = fontSetting,
      textAligns = {
        0: "left",
        1: "right",
        2: "center",
      },
      containerStyle = {
        zIndex: zIndex + defaultZIndex + 30,
        transform: `scale(${containerScale})`,
      },
      textStyle = {
        left: `${-aX}px`,
        top: `${-aY}px`,
        width: `${w}px`,
        zIndex: zIndex + defaultZIndex + 30,
        height: `${h}px`,
        opacity: `${t}`,
        transformOrigin: `${aX}px ${aY}px`,
        transform: `translate(${x}px, ${y}px) rotate(${r}deg) scaleX(${sX}) scaleY(${sY})`,
        textAlign: textAligns[textAlign],
        fontSize: `${fontSize}px`,
        lineHeight: `${h > fontSize ? fontSize : h}px`,
        color: textColor,
        fontFamily: `${font_face ? font_face.post_script_name : "MyFont"}`,
        WebkitTextStroke: `${
          material.asset.ui.strokeOverFill
            ? strokeWidth + "px" + strokeColor
            : "none"
        }`,
      };
    //设置新的样式
    let newTextStyle = { ...textStyle };

    if (!material.asset.ui.strokeOverFill && strokeWidth && strokeColor) {
      newTextStyle.zIndex = textStyle.zIndex - 1;
      newTextStyle.color = `${strokeColor}`;
      newTextStyle.top = `-${aY + strokeWidth * 1.6}px`;
      newTextStyle.WebkitTextStroke = `${strokeWidth}px ${strokeColor}`;
    }

    return (
      <div className={`${styles.poa}`} key={key} style={containerStyle}>
        <textarea
          disabled
          className={styles.textarea}
          style={textStyle}
          maxLength={maxLength}
          value={defaultText}
        />
        {/* 加一层dom进行字体遮挡 */}
        <textarea
          maxLength={maxLength}
          value={defaultText}
          readOnly={true}
          className={styles.newtextarea}
          style={newTextStyle}
        />
      </div>
    );
  };

  renderWatermarkList = (watermarkList, containerInfo) => {
    const defaultZindex = 100,
      containerStyle = {
        transform: `scale(${containerInfo})`,
      };
    return watermarkList.map((item, i) => {
      const { url, scale, rotate, position, anchor, id, width, height } = item,
        stylesObject = {
          transformOrigin: `${anchor[0]}px ${anchor[1]}px`,
          transform: `translate(${position[0]}px, ${position[1]}px) rotate(${rotate}deg) scaleX(${scale[0]}) scaleY(${scale[1]})`,
          width: width,
          height: height,
          left: `${-anchor[0]}px`,
          top: `${-anchor[1]}px`,
        };
      containerStyle.zIndex = defaultZindex + i;
      return (
        <div
          key={id}
          style={containerStyle}
          className={`${styles.watermarkContainer}`}
        >
          <img
            className={`${styles.watermarkItem}`}
            style={stylesObject}
            src={url}
            alt="watermarkImage"
          />
        </div>
      );
    });
  };

  render() {
    let {
        props,
        renderAccessory,
        renderImageAsset,
        renderTextAsset,
        renderWatermarkList,
      } = this,
      {
        backgroundImage,
        prospectImage,
        allMaterial,
        assetList,
        containerScale,
        watermarkList,
      } = props;

    return (
      <>
        {renderWatermarkList(watermarkList, containerScale)}
        {renderAccessory(styles.backgroundImage, backgroundImage)}
        {renderAccessory(styles.prospectImage, prospectImage)}
        {renderPreviewAsset(
          allMaterial,
          containerScale,
          assetList,
          renderImageAsset,
          renderTextAsset,
        )}
      </>
    );
  }
}

export default CommonPreview;
