import React, { PureComponent } from "react";
import styles from "./index.less";
import Preview from "./Preview";
import CommonPreview from "./CommonPreview";

class Group extends PureComponent {
  maxWidth = 228;

  constructor(props) {
    super(props);
    this.state = {
      groupRef: null,
      isShowPreview: false,
      previewLeft: 0,
      previewTop: 0,
    };
  }

  getInnerContainerStyles = options => {
    const style = {};

    // if (options.height) {
    //   style["height"] = `${options.height}px`;
    // }

    // if (options.width) {
    //   style["width"] = `${options.width}px`;
    // }

    // return style;
    const { originWidth, originHeight, scale } = options;
    return {
      transform: `scale(${scale})`,
      transformOrigin: "0px 0px",
      width: `${originWidth}px`,
      height: `${originHeight}px`,
    };
  };

  getContainerStyles = options => {
    const style = {};

    if (options.width) {
      style["width"] = `${options.width}px`;
      style["flex"] = `0 0 ${options.width}px`;
    }

    return style;
  };

  renderPreview = () => {
    const { isShowPreview, previewLeft, previewTop } = this.state,
      {
        groupIndex,
        parseManage,
        backgroundImage,
        prospectImage,
        allMaterial,
        assetList,
        watermarkList,
      } = this.props;

    return (
      <Preview
        watermarkList={watermarkList}
        groupIndex={groupIndex}
        parseManage={parseManage}
        backgroundImage={backgroundImage}
        prospectImage={prospectImage}
        allMaterial={allMaterial}
        assetList={assetList}
        isShow={isShowPreview}
        left={previewLeft}
        top={previewTop}
      />
    );
  };

  handleContainerMouseEnter = () => {
    const { groupRef, isShowPreview } = this.state;
    if (!groupRef || isShowPreview) return;
    this.setState({
      isShowPreview: true,
    });
  };

  handleContainerMouseLeave = () => {
    if (this.state.isShowPreview) {
      this.setState({
        isShowPreview: false,
      });
    }
  };

  componentDidUpdate() {
    const { groupRef, previewLeft, previewTop } = this.state;
    if (groupRef) {
      const { x, y, width } = groupRef.getBoundingClientRect(),
        centerX = x + width / 2;
      if (previewLeft !== centerX || previewTop !== y - 35) {
        this.setState({
          previewLeft: centerX,
          previewTop: y - 35,
        });
      }
    }
  }

  getActiveClass = isActive => {
    return isActive ? styles.maskActive : "";
  };

  getEditAfterCount = () => {
    const { originAllMaterial, allMaterial, originAssetList, assetList } = this.props;
    let count = 0;
    if (originAllMaterial && originAllMaterial.length) {
      originAllMaterial.map((originMaterial, i) => {
        const { assetName, fontSetting } = originMaterial;

        switch (originMaterial.type) {
          case 1:
            if (originAssetList[assetName] !== assetList[assetName]) {
              count++;
            }
            break;

          default:
            const defaultText =
              allMaterial && allMaterial[i] && allMaterial[i]["fontSetting"]
                ? allMaterial[i]["fontSetting"]["defaultText"]
                : "";
            if (defaultText !== fontSetting["defaultText"]) {
              count++;
            }
            break;
        }
      });
    }
    return count;
  };

  render() {
    let {
        props,
        getContainerStyles,
        getInnerContainerStyles,
        maxWidth,
        getEditAfterCount,
        getActiveClass,
        handleContainerMouseEnter,
        renderPreview,
        handleContainerMouseLeave,
      } = this,
      {
        groupIndex,
        parseManage,
        groupHeight,
        backgroundImage,
        prospectImage,
        allMaterial,
        assetList,
        isActive,
        handelGroupClick,
        watermarkList,
      } = props,
      containerInfo = parseManage.getWidthScaleByGroup(groupHeight, groupIndex);

    if (containerInfo.width > maxWidth) {
      containerInfo = parseManage.getHeightScaleByGroup(maxWidth, groupIndex);
    }
    const containerStyles = getContainerStyles(containerInfo),
      maskActive = getActiveClass(isActive),
      innerContainerStyles = getInnerContainerStyles(containerInfo);
    let maskStyles = {
      fontSize: `${24 / containerInfo.scale}px`,
    };
    return (
      <>
        <div
          className={`cursorPointer ${styles.groupContainer}`}
          style={containerStyles}
          onClick={handelGroupClick}
          onMouseEnter={handleContainerMouseEnter}
          onMouseLeave={handleContainerMouseLeave}
          ref={ref => this.setState({ groupRef: ref })}
        >
          <div className={`flex ${styles.mask} ${maskActive}`}>
            {getEditAfterCount()}/{allMaterial.length}
          </div>
          <div className={`flex ${styles.groupIndex}`}>{groupIndex}</div>
          <div className={`${styles.innerGroupContainer}`} style={innerContainerStyles}>
            <CommonPreview
              containerScale={containerInfo.scale}
              backgroundImage={backgroundImage}
              prospectImage={prospectImage}
              assetList={assetList}
              allMaterial={allMaterial}
              watermarkList={watermarkList}
            />
          </div>
        </div>
        {renderPreview()}
      </>
    );
  }
}

export default Group;
