import React, { PureComponent } from "react";
import styles from "./index.less";
import { connect } from "dva";
import Group from "./Group";
import Scrollbars from "react-custom-scrollbars";
import { getEditContainerRectInfo } from "utils/lib/getEditContainerRectInfo";
// import { getProductInfo, getTemplateInfo } from "services/workBench";

class EditGroups extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      scrollbarRef: null,
    };
  }

  handelGroupClick = (groupIndex, allMaterial) => {
    const { dispatch, parseManage, editMainPdding } = this.props,
      editContainerInfo = getEditContainerRectInfo(
        parseManage,
        groupIndex,
        editMainPdding.levelSpace + editMainPdding.customSpace + 300,
        editMainPdding.verticalSpace + editMainPdding.customSpace,
      );
    dispatch({
      type: "workBench/save",
      payload: {
        isFullEditMain: false, //避免切换比例为200%时，切换间距显示问题
        editCurrentGroupKey: groupIndex,
        isShowTransform: false,
        editWindowCurrentIndex: 1,
        isShowWatermarkTransform: false,
        currentMaterialKey: "",
        currentWatermarkKey: "",
        editMainContainerInfo: editContainerInfo,
        editMainSizePercentage: 1,
      },
    });
  };

  getRenderProps = () => {
    const { isOpenMaterialList, isOpenEditWindow } = this.props;

    return {
      containerWithPosition: isOpenMaterialList
        ? styles.withRight
        : styles.withLeft,
      isOpenEditWindow: isOpenEditWindow
        ? styles.openEditWindow
        : styles.closeEditWindow,
    };
  };

  renderGroup = (groupItem) => {
    const groupIndex = groupItem.i,
      group = groupItem.group,
      { handelGroupClick, props } = this,
      {
        parseManage,
        groupHeight,
        backgroundImageList,
        prospectImageList,
        allMaterialByGroup,
        assetList,
        editCurrentGroupKey,
        handleOriginVideoPreview,
        allOriginMaterialByGroup,
        originAssetList,
        watermarkList,
      } = props,
      originAllMaterial = allOriginMaterialByGroup[groupIndex],
      allMaterial = allMaterialByGroup[groupIndex];

    if (JSON.stringify(parseManage) === "{}" || !parseManage.groups) {
      return;
    }
    return (
      // <>
      <Group
        watermarkList={watermarkList}
        key={groupIndex}
        groupIndex={groupIndex}
        group={group}
        parseManage={parseManage}
        groupHeight={groupHeight}
        backgroundImage={
          backgroundImageList[
            parseManage.groups &&
            parseManage.groups[groupIndex] &&
            parseManage.groups[groupIndex]["backgroundImage"]
              ? parseManage.groups[groupIndex]["backgroundImage"]
              : ""
          ]
        }
        prospectImage={
          prospectImageList[
            parseManage.groups &&
            parseManage.groups[groupIndex] &&
            parseManage.groups[groupIndex]["prospectImage"]
              ? parseManage.groups[groupIndex]["prospectImage"]
              : ""
          ]
        }
        allMaterial={allMaterial}
        originAllMaterial={originAllMaterial}
        assetList={assetList}
        originAssetList={originAssetList}
        isActive={editCurrentGroupKey === groupIndex}
        handelGroupClick={handelGroupClick.bind(this, groupIndex, allMaterial)}
      />
      // </>
    );
  };

  renderGroupList = (groups) => {
    var list = [];
    for (let key in groups) {
      if (groups.hasOwnProperty(key)) {
        list.push(this.renderGroup({ i: key, group: groups[key] }));
      }
    }
    return list;
  };
  //原视频预览
  handlebuttonClick = async () => {
    // const { dispatch } = this.props,
    //   { location } = history,
    //   { product_id } = location.query,
    //   {
    //     data: { product },
    //   } = await getProductInfo({ product_id }),
    //   { template_id } = product,
    //   {
    //     data: { template },
    //   } = await getTemplateInfo({ template_id });

    // dispatch({
    //   type: "workBench/save",
    //   payload: {
    //     isShowVideoPreview: true,
    //     videoPreviewUrl: template["template_demo_video"],
    //   },
    // });
  };
  handleIconClick = (isBack) => {
    const { scrollbarRef } = this.state,
      { clientWidth, scrollLeft } = scrollbarRef.getValues();
    let left = scrollLeft + clientWidth;
    if (isBack) {
      left -= clientWidth * 2;
    }
    scrollbarRef.scrollLeft(left);
  };

  componentDidUpdate(prevProps) {
    const { scrollbarRef } = this.state;
    if (!scrollbarRef) return;
    if (
      prevProps.isOpenMaterialList === this.props.isOpenMaterialList &&
      prevProps.isOpenEditWindow === this.props.isOpenEditWindow
    )
      return;

    const left = scrollbarRef.getScrollLeft();
    scrollbarRef.scrollLeft(left + 1);
    setTimeout(() => {
      scrollbarRef.scrollLeft(left);
    }, 200);
  }
  renderGroupCover = () => {
    return (
      <div className={styles.groupCover}>
        <div className={styles.groupMask}>
          封面
          <br />
          修改
        </div>
        <div
          className={styles.groupCoverImage}
          style={{
            backgroundImage: `url(https://img1.baidu.com/it/u=1977046168,368269341&fm=26&fmt=auto&gp=0.jpg)`,
          }}
        />
      </div>
    );
  };

  render() {

    var {
        getRenderProps,
        renderGroupList,
        handleIconClick,
        handlebuttonClick,
        renderGroupCover,
        props,
      } = this,
      { allMaterialByGroup } = props,
      { isOpenEditWindow, containerWithPosition } = getRenderProps();

    return (
      <div
        className={`flex ${styles.editGroupsContainer} ${containerWithPosition} ${isOpenEditWindow}`}
      >
        <div
          className={styles.originalVideoPreview}
          onClick={handlebuttonClick.bind(this)}
        >
          <span className={`iconfont icon-bofang2 ${styles.originPlayIcon}`} />
        </div>

        {/* 左箭头 */}
        <div
          onClick={handleIconClick.bind(this, true)}
          className={`flex cursorPointer ${styles.preActionContainer}`}
        >
          <svg
            className="icon"
            viewBox="0 0 1024 1024"
            version="1.1"
            xmlns="http://www.w3.org/2000/svg"
            width="128"
            height="128"
          >
            <path
              className={`${styles.svgBackground}`}
              d="M512.113778 0C230.551704 0 0 229.034667 0 512.056889 0 793.581037 228.939852 1024 512 1024s512-228.920889 512-511.943111C1024.113778 230.513778 795.173926 0 512.113778 0z"
            />
            <path
              d="M388.361481 513.289481l207.132445 222.625186a14.696296 14.696296 0 0 0 21.807407 0 17.483852 17.483852 0 0 0 0-23.438223l-197.025185-211.759407L614.058667 292.408889a16.933926 16.933926 0 0 0 0-22.755556 14.222222 14.222222 0 0 0-21.18163 0L391.623111 485.963852c-0.796444 0.83437-1.175704 1.85837-1.744592 2.844444-0.474074 0.379259-1.061926 0.568889-1.517038 1.061926a17.464889 17.464889 0 0 0 0 23.419259z"
              fill="#FFFFFF"
            />
            <path
              d="M617.301333 712.476444a17.483852 17.483852 0 0 1 0 23.457186 14.696296 14.696296 0 0 1-21.807407 0L388.361481 513.289481a17.464889 17.464889 0 0 1 0-23.419259 3.887407 3.887407 0 0 1 0.948149-0.682666c0.18963-0.113778 0.379259-0.227556 0.568889-0.37926l0.474074-0.853333a8.419556 8.419556 0 0 1 1.270518-1.991111L592.858074 269.653333a14.222222 14.222222 0 0 1 21.18163 0 16.933926 16.933926 0 0 1 0 22.755556L420.276148 500.717037l197.025185 211.759407zM431.217778 500.717037l192 206.392889a25.467259 25.467259 0 0 1 0 34.190222 22.679704 22.679704 0 0 1-33.621334 0.037926l-207.075555-222.606222 5.840592-5.442371-5.935407 5.347556a25.467259 25.467259 0 0 1 0-34.133333l0.075852-0.056889a10.581333 10.581333 0 0 1 1.270518-1.175704 14.260148 14.260148 0 0 1 1.991112-2.768592l201.197037-216.234667a22.20563 22.20563 0 0 1 32.995555 0c8.647111 9.348741 8.628148 24.272593-0.075852 33.60237l-188.681481 202.846815z"
              fill="#FFFFFF"
            />
          </svg>
        </div>
        <Scrollbars ref={(ref) => this.setState({ scrollbarRef: ref })}>
          <div className={`flex ${styles.groupContainer}`}>
            {/* {renderGroupCover()} */}
            {renderGroupList(allMaterialByGroup)}
          </div>
        </Scrollbars>
        {/* 右箭头 */}
        <div
          onClick={handleIconClick.bind(this, false)}
          className={`flex cursorPointer ${styles.nextActionContainer}`}
        >
          <svg
            className="icon"
            viewBox="0 0 1024 1024"
            version="1.1"
            xmlns="http://www.w3.org/2000/svg"
            width="128"
            height="128"
          >
            <path
              className={`${styles.svgBackground}`}
              d="M511.886222 1024C793.448296 1024 1024 794.965333 1024 511.943111 1024 230.4 795.060148 0 512 0S0 228.920889 0 511.943111C-0.113778 793.467259 228.826074 1024 511.886222 1024z"
            />
            <path
              d="M635.638519 510.710519l-207.132445-222.625186a14.696296 14.696296 0 0 0-21.807407 0 17.483852 17.483852 0 0 0 0 23.41926l197.025185 211.77837L409.960296 731.591111a16.933926 16.933926 0 0 0 0 22.755556 14.222222 14.222222 0 0 0 21.18163 0l201.234963-216.310519c0.777481-0.83437 1.175704-1.85837 1.744592-2.844444 0.474074-0.379259 1.061926-0.568889 1.517038-1.061926a17.464889 17.464889 0 0 0 0-23.419259z"
              fill="#FFFFFF"
            />
            <path
              d="M406.71763 311.504593a17.483852 17.483852 0 0 1 0-23.438223 14.696296 14.696296 0 0 1 21.788444 0L635.638519 510.710519a17.464889 17.464889 0 0 1 0 23.419259 3.887407 3.887407 0 0 1-0.948149 0.682666c-0.18963 0.113778-0.379259 0.227556-0.568889 0.37926l-0.474074 0.853333a8.419556 8.419556 0 0 1-1.270518 1.991111L431.141926 754.346667a14.222222 14.222222 0 0 1-21.18163 0 16.933926 16.933926 0 0 1 0-22.755556l193.763556-208.308148-197.025185-211.77837z m186.064592 211.77837L400.782222 316.890074a25.467259 25.467259 0 0 1 0-34.190222 22.679704 22.679704 0 0 1 33.621334-0.037926l207.075555 222.606222-5.840592 5.442371 5.935407-5.347556a25.467259 25.467259 0 0 1 0 34.133333l-0.075852 0.056889a10.581333 10.581333 0 0 1-1.270518 1.175704 14.260148 14.260148 0 0 1-1.991112 2.768592l-201.197037 216.234667a22.20563 22.20563 0 0 1-32.995555 0 24.917333 24.917333 0 0 1 0.075852-33.60237l188.681481-202.846815z"
              fill="#FFFFFF"
            />
          </svg>
        </div>
      </div>
    );
  }
}

const stateToProps = (state) => {
  const { workBench } = state,
    {
      isOpenMaterialList,
      allMaterialByGroup,
      parseManage,
      groupHeight,
      backgroundImageList,
      prospectImageList,
      assetList,
      editCurrentGroupKey,
      editMainPdding,
      isOpenEditWindow,
      allOriginMaterialByGroup,
      originAssetList,
      watermarkList,
    } = workBench;
  return {
    assetList: assetList,
    parseManage: parseManage,
    groupHeight: groupHeight,
    isOpenMaterialList: isOpenMaterialList,
    backgroundImageList: backgroundImageList,
    prospectImageList: prospectImageList,
    allMaterialByGroup: allMaterialByGroup,
    editCurrentGroupKey: editCurrentGroupKey,
    editMainPdding: editMainPdding,
    isOpenEditWindow: isOpenEditWindow,
    allOriginMaterialByGroup: allOriginMaterialByGroup,
    originAssetList: originAssetList,
    watermarkList: watermarkList,
  };
};

export default connect(stateToProps)(EditGroups);
