import React, { PureComponent } from "react";
import Moveable from "react-moveable";

class Transform extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      moveableRef: null,
    };
  }

  onDragStart = (dragStartProps) => {
    const { handleDragStart } = this.props;
    handleDragStart && handleDragStart(dragStartProps);
  };

  onDrag = (dragProps) => {
    dragProps.target.style.transform = dragProps.transform;
    const { handleDrag } = this.props;
    handleDrag && handleDrag(dragProps);
  };

  onDragEnd = (dragEndProps) => {
    const { handleDragEnd } = this.props;
    handleDragEnd && handleDragEnd(dragEndProps);
  };

  onScaleStart = (scaleStartProps) => {
    const { handleScaleStart } = this.props;
    handleScaleStart && handleScaleStart(scaleStartProps);
  };

  onScale = (scaleProps) => {
    scaleProps.target.style.transform = scaleProps.transform;
    const { handleScale } = this.props;
    handleScale && handleScale(scaleProps);
  };

  onScaleEnd = (scaleEndProps) => {
    const { handleScaleEnd } = this.props;
    handleScaleEnd && handleScaleEnd(scaleEndProps);
  };

  onRotateStart = (rotateStartProps) => {
    const { handleRotateStart } = this.props;
    handleRotateStart && handleRotateStart(rotateStartProps);
  };

  onRotate = (rotateProps) => {
    rotateProps.target.style.transform = rotateProps.transform;
    const { handleRotate } = this.props;

    handleRotate && handleRotate(rotateProps);
  };

  onRotateEnd = (rotateEndProps) => {
    const { handleRotateEnd } = this.props;
    handleRotateEnd && handleRotateEnd(rotateEndProps);
  };

  componentDidUpdate() {
    const {
        moveableRef: { moveable },
      } = this.state,
      { target } = this.props;
    moveable.setState({
      target: { ...target },
    });
  }
  render() {
    const {
        onDragStart,
        onDrag,
        onDragEnd,
        onScaleStart,
        onScale,
        onScaleEnd,
        onRotateStart,
        onRotate,
        onRotateEnd,
      } = this,
      { disableRotatable, disableScaleble } = this.props;

    let rotatable = true,
      scalable = true;
    if (disableRotatable) {
      rotatable = false;
    }
    if (disableScaleble) {
      scalable = false;
    }

    return (
      <Moveable
        ref={(ref) => this.setState({ moveableRef: ref })}
        target={this.props.target} //拖动蓝色框显示的目标
        container={null}
        dragTarget={this.props.bigtarget} //拖动可移动目标的目标
        origin={false}
        passDragArea={true}
        keepRatio={true}
        warpable={true}
        scalable={scalable}
        draggable={true}
        rotatable={rotatable}
        checkInput={true} //若为可编辑的元素，比如textarea 时阻止拖动
        className={"moveableCustom"}
        renderDirections={["nw", "ne", "sw", "se"]}
        // renderDirections={["n", "nw", "ne", "s", "se", "sw", "e", "w"]}
        onDragStart={onDragStart}
        onDrag={onDrag}
        onDragEnd={onDragEnd}
        onScaleStart={onScaleStart}
        onScale={onScale}
        onScaleEnd={onScaleEnd}
        onRotateStart={onRotateStart}
        onRotate={onRotate}
        onRotateEnd={onRotateEnd}
      />
    );
  }
}

export default Transform;
