import React, { PureComponent } from "react";
import styles from "./index.less";
import { connect } from "dva";
import { getEditContainerRectInfo } from "utils/lib/getEditContainerRectInfo";
import { renderPreviewAsset } from "utils/lib/renderPreviewAsset";
import Moveable from "./Moveable";
import Transform from "./Transform";
import { getIndexByAssetName } from "utils/lib/mapUtils";
import Affine from "utils/lib/Affine";
// import DragImageList from "./DragImageList";
import { message, Spin } from "antd";
import Upload from "utils/lib/Upload";
import env from "@/env";
import Cookie from "js-cookie";
import PubSub from "pubsub-js";
import { TOKEN_NAME } from "@/types";
import { isVideo, wipeOffQueryString } from "utils/index";
import app from "@/utils/lib/app";
import MatrixUtil from "utils/lib/MatrixUtil";

class EditMain extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      materialRefList: {},
      editBoxRefList: {},
      groupContainerRefList: {},
      maskCanvasDom: null,
      uploadFileRef: null,
      fileIdCount: 0,
      watermarkRefList: {},
      currentImageAssetChange: null,
      bigtarget: "",
    };
    this.typeList = ["JPG", "JPEG", "MP4", "GIF", "PNG"];
    this.backwindowStatus = true;
    this.equlist = [];
    this.imageIndex = 0;
    this.multilayerBackground = false;
    this.rotatex = 0;
    this.rotatey = 0;
    this.defaultZIndex = 0;
    this.frame = {
      referPoint: {
        x: 0,
        y: 0,
      },
      referScale: {
        x: 1,
        y: 1,
      },
      referRotate: 0,
    };

    this.watermarkFrame = {
      referPoint: {
        x: 0,
        y: 0,
      },
      referScale: {
        x: 1,
        y: 1,
      },
      referRotate: 0,
    };

    // 编辑器初始化
    // 1. 获取当前组和素材
    // 2. 根据当前组初始化组容器大小
    // 3. 遍历组内所有替换元素，前景图，背景图
    // 4. 按照层级顺序摆放顺序

    // 切换组逻辑
    // 1. 获取当前组信息
    // 2. 重新初始化组编辑器

    // 选中编辑素材逻辑
    // 1. 已知当前组
    // 2. 根据位置判断当前选中的素材
    // 3. 对素材进行编辑

    // 同时只会存在一个可编辑组，一个选中的可编辑素材
    // 只有图片才会有拖动框
  }

  handleClipImageOnOk = async (file) => {
    const {
      dispatch,
      assetList,
      editCurrentGroupKey,
      allMaterialByGroup,
      currentMaterialKey,
      allOriginMaterialByGroup,
      mattingObj,
    } = this.props;
    let { origin_draft, cut_origin_draft } = this.props;
    let upload = new Upload({
      getTokenUrl: env.getTokenUrl,
      jwt: Cookie.get(TOKEN_NAME),
    }),
      cropImageUrl = await upload.uploadFile(file),
      isVideoFile = isVideo(wipeOffQueryString(cropImageUrl));
    if (isVideoFile) {
      cropImageUrl += "?x-oss-process=video/snapshot,t_0";
    }

    const fileName = file.name,
      fileNamePoint = fileName.lastIndexOf("."),
      size = Math.floor(file.size / 1024 / 1024),
      index = this.typeList.indexOf(
        fileName.substr(fileNamePoint + 1).toUpperCase(),
      );

    if (index === -1) {
      message.info("文件类型错误");
      dispatch({
        type: "workBench/save",
        payload: {
          editMainLoading: false,
        },
      });
      return;
    }

    if (!isVideoFile && size >= 12) {
      dispatch({
        type: "workBench/save",
        payload: {
          editMainLoading: false,
        },
      });
      message.info("仅支持12M以内图片或50M以内视频");
      return;
    }

    if (isVideoFile && size >= 50) {
      dispatch({
        type: "workBench/save",
        payload: {
          editMainLoading: false,
        },
      });
      message.info("仅支持12M以内图片或50M以内视频");
      return;
    }

    if (JSON.stringify(origin_draft) !== "{}") {
      origin_draft[currentMaterialKey] = cropImageUrl;
    }
    if (JSON.stringify(cut_origin_draft) !== "{}") {
      cut_origin_draft[currentMaterialKey] = cropImageUrl;
    }
    mattingObj[currentMaterialKey] = "";

    dispatch({
      type: "workBench/replaceImage",
      payload: {
        assetName: currentMaterialKey,
        assetList: assetList,
        origin_draft: origin_draft,
        cut_origin_draft: cut_origin_draft,
        editCurrentGroupKey: editCurrentGroupKey,
        allMaterialByGroup: allMaterialByGroup,
        allOriginMaterialByGroup: allOriginMaterialByGroup,
        dataItem: {
          imageSrc: cropImageUrl,
        },
      },
    });
    dispatch({
      type: "workBench/save",
      payload: {
        mattingObj,
      },
    });
  };

  getCurrentMaterial = () => {
    const {
      allMaterialByGroup,
      editCurrentGroupKey,
      currentMaterialKey,
    } = this.props,
      allMaterialList = allMaterialByGroup[editCurrentGroupKey],
      index = getIndexByAssetName(currentMaterialKey, allMaterialList);

    return allMaterialList[index];
  };

  getCurrentWatermark = () => {
    const { watermarkList, currentWatermarkKey } = this.props,
      index = this.getWatermark(watermarkList, currentWatermarkKey);
    return watermarkList[index];
  };

  getWatermark = (watermarkList, currentWatermarkKey) => {
    let index = 0;
    watermarkList.map((item, i) => {
      if (item.id === currentWatermarkKey) {
        index = i;
      }
    });

    return index;
  };

  setMaterialRef = (ref, key) => {
    const { materialRefList } = this.state;
    if (ref && materialRefList[key] !== ref) {
      const newRefList = {
        ...materialRefList,
        [key]: ref,
      };
      this.setState({
        materialRefList: newRefList,
      });
    }
  };

  setContainerRef = (ref, key) => {
    const { editBoxRefList } = this.state;
    if (ref && editBoxRefList[key] !== ref) {
      const newRefList = {
        ...editBoxRefList,
        [key]: ref,
      };
      this.setState({
        editBoxRefList: newRefList,
      });
    }
  };

  setGroupContainerRef = (ref, key) => {
    if (key == "") {
      return;
    }
    const { groupContainerRefList } = this.state;
    if (ref && groupContainerRefList[key] !== ref) {
      const newRefList = {
        ...groupContainerRefList,
        [key]: ref,
      };
      this.setState({
        groupContainerRefList: newRefList,
      });
    }
  };

  setWatermarkRef = (ref, key) => {
    this.setState((state) => {
      state.watermarkRefList[key] = ref;
      return {
        ...state,
      };
    });
  };

  // renderAccessory = (className, image) => {
  //   if (!image) return null;
  //   return (
  //     <div
  //       className={`${styles.accessoryImage} ${className}`}
  //       style={{
  //         backgroundImage: `url(${image})`,
  //       }}
  //     />
  //   );
  // };
  renderBackgroundList = (i, type, zindex, src) => {
    return (
      <img
        key={i}
        className={styles.backgroundsImage}
        type={type}
        draggable="false"
        style={{ zIndex: zindex }}
        src={src}
        alt=""
      />
    );
  };

  renderBackgroundTwoList = (i, v, allBackgroundList) => {
    if (v.f && v.b) {
      return (
        <div key={i}>
          <img
            className={styles.backgroundsImage}
            type="背景图"
            draggable="false"
            style={{ zIndex: v.index }}
            src={allBackgroundList[v.b]}
            alt=""
            ƒ
          />
          <img
            className={styles.backgroundsImage}
            type="前景图"
            draggable="false"
            style={{ zIndex: v.index + 1 }}
            src={allBackgroundList[v.f]}
            alt=""
          />
        </div>
      );
    } else {
      return (
        <img
          key={i}
          className={styles.backgroundsImage}
          type={v.b ? "背景图" : "前景图"}
          draggable="false"
          style={{ zIndex: v.b ? v.index : v.index + 1 }}
          src={v.b ? allBackgroundList[v.b] : allBackgroundList[v.f]}
          alt=""
        />
      );
    }
  };
  //背景图和前景图，层级叠加
  renderAccessory = () => {
    const {
      backgroundImageList,
      parseManage,
      editCurrentGroupKey,
      prospectImageList,
    } = this.props;
    if (!parseManage.groups) {
      return;
    }
    const { renderBackgroundTwoList } = this;
    let allBackgroundList = Object.assign(
      {},
      backgroundImageList,
      prospectImageList,
    );
    let sourceHost = env.sourceHost
    for (var key in allBackgroundList) {
      if (allBackgroundList.hasOwnProperty(key)) {
        if (allBackgroundList[key].indexOf("//") === -1) {
          allBackgroundList[key] = sourceHost + allBackgroundList[key];
        } else {
          allBackgroundList[key] += "?cache=1024";
        }

        let fileFormat = key.substring(key.lastIndexOf(".") + 1).toUpperCase();
        if (fileFormat === "MP4" || fileFormat === "MOV") {
          if (allBackgroundList[key].indexOf("static.atvideo.cc") !== -1) {
            if(allBackgroundList[key].indexOf("?cache=1024") !== -1){
              allBackgroundList[key] += "&vframe/png/offset/0";
            }else{
              allBackgroundList[key] += "?vframe/png/offset/0";
            }
          } else {
            if(allBackgroundList[key].indexOf("?cache=1024") !== -1){
              allBackgroundList[key] += "&x-oss-process=video/snapshot,t_0";
            }else{
              allBackgroundList[key] += "?x-oss-process=video/snapshot,t_0";
            }
          }
        }

      }
    }
    let allImageList = parseManage.groups[editCurrentGroupKey].allMaterial;

    return (
      <div className={styles.backgroundImageWrap}>
        {allImageList.map((v, i) => {
          if (v.asset.ui.b || v.asset.ui.f) {
            return renderBackgroundTwoList(i, v.asset.ui, allBackgroundList);
          }
        })}
      </div>
    );
  };

  getBackgroundAccessorySrc = () => {
    const {
      backgroundImageList,
      parseManage,
      editCurrentGroupKey,
    } = this.props;

    if (editCurrentGroupKey) {
      return backgroundImageList[
        parseManage.groups[editCurrentGroupKey]["backgroundImage"]
      ];
    }

    return "";
  };

  getProspectAccessorySrc = () => {
    const { prospectImageList, parseManage, editCurrentGroupKey } = this.props;

    if (editCurrentGroupKey) {
      return prospectImageList[
        parseManage.groups[editCurrentGroupKey]["prospectImage"]
      ];
    }

    return "";
  };

  getContainerInfo = () => {
    const { editMainContainerInfo } = this.props;
    // editMainContainerInfo.scale = 1;
    return editMainContainerInfo;
  };

  getContainerStyle = (options) => {
    const { width, height, scale } = options;
    return {
      transform: `scale(${scale})`,
      width: `${width}px`,
      height: `${height}px`,
    };
  };

  handleImageMaterialClick = (key) => {
    const { dispatch } = this.props;
    const { materialRefList } = this.state;
    for (var i in materialRefList) {
      materialRefList[i].blur();
    }
    dispatch({
      type: "workBench/save",
      payload: {
        editWindowCurrentIndex: 2,
        isShowTransform: true,
        isShowWatermarkTransform: false,
        currentMaterialKey: key,
      },
    });
  };

  replaceMultipleImage = () => {
    const {
      currentSelectImageList,
      dispatch,
      assetList,
      editCurrentGroupKey,
      allMaterialByGroup,
      dragImageMoveing,
      allOriginMaterialByGroup,
      origin_draft,
      cut_origin_draft,
    } = this.props;
    const isMultiple = currentSelectImageList.length > 1;
    if (!dragImageMoveing || !isMultiple) return;

    dispatch({
      type: "workBench/replaceMultipleImage",
      payload: {
        assetList: assetList,
        editCurrentGroupKey: editCurrentGroupKey,
        allMaterialByGroup: allMaterialByGroup,
        currentSelectImageList: currentSelectImageList,
        allOriginMaterialByGroup: allOriginMaterialByGroup,
        origin_draft,
        cut_origin_draft,
      },
    });
    dispatch({
      type: "workBench/save",
      payload: {
        currentSelectImageList: [],
      },
    });
  };

  handleImageMaterialMouseUp = (key) => {
    const {
      dragImageMoveing,
      currentSelectImageList,
      dispatch,
      assetList,
      editCurrentGroupKey,
      allMaterialByGroup,
      selectMaterialType,
      allOriginMaterialByGroup,
    } = this.props;
    //如果是水印则不触发图库的上传
    if (selectMaterialType == 2) {
      return;
    }
    let { origin_draft, cut_origin_draft } = this.props;
    if (!dragImageMoveing) return;
    const isMultiple = currentSelectImageList.length > 1;
    if (isMultiple) {
      this.replaceMultipleImage();
    } else {
      origin_draft[key] = currentSelectImageList[0].imageSrc;
      cut_origin_draft[key] = currentSelectImageList[0].imageSrc;
      dispatch({
        type: "workBench/replaceImage",
        payload: {
          assetName: key,
          assetList: assetList,
          origin_draft: origin_draft,
          cut_origin_draft,
          allOriginMaterialByGroup: allOriginMaterialByGroup,
          editCurrentGroupKey: editCurrentGroupKey,
          allMaterialByGroup: allMaterialByGroup,
          dataItem: currentSelectImageList[0],
        },
      });
    }
    //图片替换成功后也要初始化原选中的数据
    PubSub.publish("currentSelectImageChange");
    dispatch({
      type: "workBench/save",
      payload: {
        currentSelectImageList: [],
      },
    });
  };

  handleEditContainerMouseUp = (e) => {
    e.preventDefault();
    let nowImageKey = this.getCordPictureKey(e);
    // 针对多图层拖拽上传图片处理
    if (nowImageKey) {
      this.handleImageMaterialMouseUp(nowImageKey);
    }

    if (e.target.tagName !== "IMG") {
      this.replaceMultipleImage();
    }
  };

  handleImageMaterialDoubleClick = (key) => {
    const { uploadFileRef } = this.state;
    const { dispatch } = this.props;
    if (uploadFileRef) {
      const cb = (key, e) => {
        const file = e.target.files[0];
        if (!file) return;
        //这里的时候loading
        dispatch({
          type: "workBench/save",
          payload: {
            editMainLoading: true,
          },
        });
        this.handleClipImageOnOk(file);

        uploadFileRef.value = null;
      };
      this.setState(
        {
          currentImageAssetChange: cb.bind(this, key),
        },
        () => {
          uploadFileRef.click();
        },
      );
    }
  };

  // 获取基于网页的全局坐标
  getGlobalPos = (clientX, clientY) => {
    var scrollTop =
      document.documentElement.scrollTop ||
      window.pageYOffset ||
      document.body.scrollTop;
    var scrollLeft =
      document.documentElement.scrollLeft ||
      window.pageXOffset ||
      document.body.scrollLeft;

    // console.log(document.documentElement.scrollLeft, window.pageXOffset, document.body.scrollLeft)

    return { x: clientX + scrollLeft, y: clientY + scrollTop };
  };

  getGroupPos = (x, y) => {
    let { x: clientX, y: clientY } = this.getGlobalPos(x, y);

    let { editCurrentGroupKey } = this.props;
    let { groupContainerRefList } = this.state;

    let groupBox = groupContainerRefList[editCurrentGroupKey];

    let groupBoxBound = groupBox.getBoundingClientRect();

    // 组编辑框据矩阵 group
    // 素材编辑框矩阵 container

    let containerInfo = this.getContainerInfo();

    let groupMatrixVar = {
      position: {
        x: groupBoxBound.x, // + containerInfo.width * containerInfo.scale / 2,
        y: groupBoxBound.y, //+ containerInfo.height * containerInfo.scale /2
      },
      anchor: {
        x: 0, //containerInfo.originWidth / 2,
        y: 0, //containerInfo.originHeight / 2
      },
      scale: {
        x: containerInfo.scale,
        y: containerInfo.scale,
      },
      rotate: 0,
    };

    let groupMat = MatrixUtil.getTransformMatrix(groupMatrixVar);

    // console.log(groupBoxBound, groupMatrixVar);

    let position = groupMat.getInverse().transform({ x: clientX, y: clientY });

    // console.log("position:", clientX, clientY);
    return position;
  };

  // 获取编辑框的局部坐标
  getLocalPos = (e) => {
    let { x: clientX, y: clientY } = this.getGlobalPos(e.clientX, e.clientY);

    let imageAsset = this.getCurrentMaterial();
    let { editCurrentGroupKey } = this.props;
    let { groupContainerRefList } = this.state;

    let groupBox = groupContainerRefList[editCurrentGroupKey];
    // let editBox = editBoxRefList[imageAsset.assetName];

    let groupBoxBound = groupBox.getBoundingClientRect();

    // 组编辑框据矩阵 group
    // 素材编辑框矩阵 container

    let containerInfo = this.getContainerInfo();

    let groupMatrixVar = {
      position: {
        x: groupBoxBound.x, // + containerInfo.width * containerInfo.scale / 2,
        y: groupBoxBound.y, //+ containerInfo.height * containerInfo.scale /2
      },
      anchor: {
        x: 0, //containerInfo.originWidth / 2,
        y: 0, //containerInfo.originHeight / 2
      },
      scale: {
        x: containerInfo.scale,
        y: containerInfo.scale,
      },
      rotate: 0,
    };

    let { p, a, s, r } = imageAsset.asset.ui;
    let editMatrixVar = {
      anchor: a,
      position: p,
      scale: s,
      rotate: r,
    };

    let groupMat = MatrixUtil.getTransformMatrix(groupMatrixVar);
    let editMat = MatrixUtil.getTransformMatrix(editMatrixVar);
    groupMat.premultiply(editMat);

    // console.log(groupBoxBound, groupMatrixVar);

    let position = groupMat.getInverse().transform({ x: clientX, y: clientY });

    // console.log("position:", clientX, clientY);
    return position;
  };

  renderImageAsset = (imageAsset, containerScale, imageSrc) => {
    const {
      currentMaterialKey,
    } = this.props;
    const {
      defaultZIndex,
      setMaterialRef,
      setContainerRef,
      handleImageMaterialClick,
      handleImageMaterialMouseUp,
    } = this;

    // 图片渲染逻辑
    // 1. 该图片第一次处理，获取原始尺寸，和 editSize 并自动适配到中心, 如果锚点不在中心，重置矩阵到中心
    // 2. 不是第一次
    //    1). 如果图片一致，直接使用上一次的矩阵信息
    //    2). 不一致， 重置矩阵，当成第一次一致
    let { p, a, s, r, editSize } = imageAsset.asset.ui;
    let {
      position,
      anchor,
      scale,
      rotate,
      originSize,
      originImageSize,
      transparency,
      assetName: key,
      zIndex,
    } = imageAsset;

    let initMatVar = {
      anchor: a,
      position: p,
      scale: s,
      rotate: r,
    };

    let matVar = {
      anchor: anchor,
      position: position,
      scale: scale,
      rotate: rotate,
    };

    let imageSize = originImageSize ? originImageSize : originSize;
    var matrix = MatrixUtil.getTransformMatrix(matVar).toMatrixString();

    let imageStyle = {
      width: `${imageSize[0]}px`,
      height: `${imageSize[1]}px`,
      position: "absolute",
      opacity: `${transparency}`,
      border: `2px dashed #FF5172`,
      transformOrigin: `0px 0px`,
      transform: matrix,
    };
    let editMatrix = MatrixUtil.getTransformMatrix(initMatVar).toMatrixString();
    let containerStyle = {
      zIndex: zIndex + defaultZIndex,
      overflow: "hidden",
      transformOrigin: "0px 0px",
      width: editSize[0] + "px",
      height: editSize[1] + "px",
      transform: `${editMatrix}`,
    };

    if (currentMaterialKey === key) {
      delete imageStyle.border;
    }

    return (
      <div
        key={key}
        className={`${styles.poa} replaceImageDomWrap`}
        style={containerStyle}
        ref={(ref) => setContainerRef(ref, key)}
      >
        <img
          ref={(ref) => setMaterialRef(ref, key)}
          className={`replaceImageDom ${styles.poa}`}
          onClick={handleImageMaterialClick.bind(this, key)}
          onMouseUp={handleImageMaterialMouseUp.bind(this, key)}
          style={imageStyle}
          src={imageSrc}
          assname={imageAsset.assetName}
          alt="asset"
        // title="双击替换图片/视频"
        // onDoubleClick={handleImageMaterialDoubleClick.bind(this, key)}
        />
      </div>
    );
  };

  renderImageMaterial = (
    w,
    h,
    aX,
    aY,
    x,
    y,
    sX,
    sY,
    r,
    t,
    zIndex,
    containerScale,
    src,
    key,
    item,
  ) => {
    let moveleft = aX;
    if (item.asset && !item.asset.ui.a[0]) {
      aX = item.asset.ui.editSize[0] / 2;
      moveleft = 0;
    }
    const {
      defaultZIndex,
      setMaterialRef,
      handleImageMaterialClick,
      handleImageMaterialMouseUp,
      handleImageMaterialDoubleClick,
      props,
    } = this,
      { currentMaterialKey } = props,
      containerStyle = {
        zIndex: zIndex + defaultZIndex,
        transform: `scale(${containerScale})`,
      };

    var newAffine = new Affine();
    //图片的原始数据
    //改变锚点的位置为中心
    var matVar = {
      position: {
        x: item.position[0],
        y: item.position[1],
      },
      anchor: {
        x: item.anchor[0],
        y: item.anchor[1],
      },
      scale: {
        x: item.scale[0],
        y: item.scale[1],
      },
      rotate: 0,
    };
    //旋转
    // let start = {
    //   x: 0 - matVar.position.x,
    //   y: 0 - matVar.position.y
    // }
    // let end = {
    //   x: this.rotatex - matVar.position.x,
    //   y: this.rotatey - matVar.position.y
    // }

    newAffine.set(matVar.anchor, matVar.position, matVar.scale, matVar.rotate);
    // matVar.position = newAffine.getPositionFromAnchor(matVar.anchor.x, matVar.anchor.y);

    // var testAffine = this.getTransformMatrix(initVar)
    // matVar.position = testAffine.getPositionFromAnchor( matVar.anchor.x, matVar.anchor.y );

    var imageStyle = {
      left: `${-moveleft}px`,
      top: `${-aY}px`,
      width: `${w}px`,
      height: `${h}px`,
      position: "absolute",
      opacity: `${t}`,
      border: `${2 / Math.abs(sX)}px dashed #FF5172`,
      transformOrigin: `${aX}px ${aY}px`,
      transform: `translate(${x}px, ${y}px) rotate(${r}deg) scaleX(${sX}) scaleY(${sY})`,
      // transformOrigin: `0px 0px`,
      // transform: matrix,
    };

    if (currentMaterialKey === key) {
      delete imageStyle.border;
    }
    return (
      <div
        key={key}
        className={`${styles.poa} replaceImageDomWrap`}
        style={containerStyle}
      >
        <img
          ref={(ref) => setMaterialRef(ref, key)}
          className={`replaceImageDom ${styles.poa}`}
          onClick={handleImageMaterialClick.bind(this, key)}
          onMouseUp={handleImageMaterialMouseUp.bind(this, key)}
          style={imageStyle}
          src={src}
          assname={item.assetName}
          alt="asset"
        // title="双击替换图片/视频"
        // onDoubleClick={handleImageMaterialDoubleClick.bind(this, key)}
        />
      </div>
    );
  };

  handleTextMaterialInput = (e, key, maxLength, material) => {
    const value = e.target.value,
      { dispatch, editCurrentGroupKey, allMaterialByGroup } = this.props;
    let groupsList = allMaterialByGroup[editCurrentGroupKey];
    let equlist = [];
    let { assetName, asset } = material;
    groupsList.forEach((v) => {
      if (
        asset.ui &&
        asset.ui.link &&
        asset.ui.link.length &&
        asset.ui.link.indexOf(v.asset.ui.index) !== -1 &&
        assetName !== v.assetName
      ) {
        equlist.push(v.assetName);
      }
    });

    const size = app.getAutoSize(material, value);
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: value,
          size,
        },
      },
    });

    //modal中循环替换
    if (equlist.length) {
      dispatch({
        type: "workBench/changeEqualText",
        payload: {
          data: {
            text: value,
            equlist: equlist,
            size,
          },
        },
      });
    }
  };

  handleEditTextBlur = (maxLength, e) => {
    const { dispatch } = this.props;
    let val = e.target.value;
    if (val.length > maxLength) {
      message.info("文字字数超过模板的最大限制");
      val = val.substring(0, maxLength);
    }

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: val,
        },
      },
    });
  };

  handleTextMaterilaDoubleClick = (key) => {
    const { materialRefList } = this.state,
      currentTextDom = materialRefList[key];
    if (currentTextDom) {
      currentTextDom.select();
    }
  };

  handleTextMaterilaFocus = (e, key, maxLength, material) => {
    const { dispatch, editCurrentGroupKey, allMaterialByGroup } = this.props;
    let groupsList = allMaterialByGroup[editCurrentGroupKey];
    let equlist = [];
    let { assetName, asset } = material;
    groupsList.forEach((v) => {
      if (
        asset.ui &&
        asset.ui.link &&
        asset.ui.link.length &&
        asset.ui.link.indexOf(v.asset.ui.index) !== -1 &&
        assetName !== v.assetName //避免非相同字相同框内字体自适应
      ) {
        equlist.push(v.assetName);
      }
    });

    dispatch({
      type: "workBench/save",
      payload: {
        equalList: equlist,
        currentMaterialKey: key,
        isShowTransform: false,
        isShowWatermarkTransform: false,
        editWindowCurrentIndex: 3,
        currentWatermarkKey: "",
      },
    });
  };
  changeValue = () => {

  };

  renderTextAsset = (material) => {
    console.log(material);
    let {
      anchor,
      position,
      scale,
      rotate,
      fontSetting,
      zIndex,
      transparence,
      originSize,
      assetName,
    } = material;
    const {
      defaultZIndex,
      setMaterialRef,
      handleTextMaterialInput,
      handleTextMaterilaDoubleClick,
      handleTextMaterilaFocus,
      handleEditTextBlur,
    } = this,
      {
        defaultText,
        fontSize,
        textAlign,
        textColor,
        maxLength,
        strokeWidth,
        strokeColor,
        font_face,
      } = fontSetting,
      textAligns = {
        0: "left",
        1: "right",
        2: "center",
      },
      containerStyle = {
        zIndex: zIndex + defaultZIndex + 30,
        // transform: `scale(${containerScale})`,
      };
    let fontFamily = `${font_face ? font_face.post_script_name : "MyFont"}`;
    //避免出现.0 字体不显示在标签上
    if (fontFamily.indexOf(".") !== -1) {
      fontFamily = fontFamily.split(".")[0];
    }
    let textMatrixString = MatrixUtil.composeTransformMatrix(
      anchor,
      position,
      scale,
      rotate,
    ).toMatrixString();
    let textStyle = {
      width: `${originSize[0]}px`,
      height: `${originSize[1]}px`,
      zIndex: zIndex + defaultZIndex + 30,
      opacity: `${transparence}`,
      transformOrigin: `0px 0px`,
      transform: `${textMatrixString}`,
      textAlign: textAligns[textAlign],
      fontSize: `${fontSize}px`,
      lineHeight: `${originSize[1] > fontSize ? fontSize : originSize[1]}px`,
      color: textColor,
      fontFamily: fontFamily,
      // WebkitTextStroke : `${strokeWidth}px ${strokeColor}`,
      WebkitTextStroke: `${material.asset.ui.strokeOverFill
        ? strokeWidth + "px" + strokeColor
        : "none"
        }`,
    },
      textBorderStyle = {
        ...textStyle,

        borderWidth: `${2 / Math.abs(scale[0])}px`,
      };

    //设置新的样式
    let newTextStyle = { ...textStyle };
    newTextStyle.zIndex = textStyle.zIndex - 1;
    newTextStyle.color = `${strokeColor}`;
    newTextStyle.fontSize = `${fontSize}px`;
    newTextStyle.WebkitTextStroke = `${strokeWidth}px ${strokeColor}`;
    // newTextStyle.lineHeight = `100%`;
    newTextStyle.display =
      !material.asset.ui.strokeOverFill && strokeWidth && strokeColor
        ? "block"
        : "none";
    return (
      <div className={`${styles.poa}`} key={assetName} style={containerStyle}>
        <div
          className={`${styles.poa} ${styles.textareaBorderBox}`}
          style={textBorderStyle}
        />
        <textarea
          ref={(ref) => setMaterialRef(ref, assetName)}
          maxLength={maxLength}
          className={styles.textarea}
          onChange={(e) =>
            handleTextMaterialInput(e, assetName, maxLength, material)
          }
          style={textStyle}
          value={defaultText}
          onBlur={handleEditTextBlur.bind(this, maxLength)}
          onDoubleClick={handleTextMaterilaDoubleClick.bind(this, assetName)}
          onFocus={(e) =>
            handleTextMaterilaFocus(e, assetName, maxLength, material)
          }
        />

        {/* 加一层dom进行字体遮挡 */}
        {/* {!material.asset.ui.strokeOverFill && strokeWidth && strokeColor ? ( */}
        <textarea
          wrap="hard"
          maxLength={maxLength}
          value={defaultText}
          // readOnly={true}
          className={styles.newtextarea}
          style={newTextStyle}
        />
        {/* ) : (
        "" */}
        {/* )} */}
      </div>
    );
  };

  renderTextMaterial = (
    w,
    h,
    aX,
    aY,
    x,
    y,
    sX,
    sY,
    r,
    t,
    zIndex,
    containerScale,
    fontSetting,
    key,
    material,
  ) => {
    const {
      defaultZIndex,
      setMaterialRef,
      handleTextMaterialInput,
      handleTextMaterilaDoubleClick,
      handleTextMaterilaFocus,
      handleEditTextBlur,
    } = this,
      {
        defaultText,
        fontSize,
        textAlign,
        textColor,
        maxLength,
        strokeWidth,
        strokeColor,
        font_face,
      } = fontSetting,
      textAligns = {
        0: "left",
        1: "right",
        2: "center",
      },
      containerStyle = {
        zIndex: zIndex + defaultZIndex + 30,
        // transform: `scale(${containerScale})`,
      },
      textStyle = {
        left: `${-aX}px`,
        top: `${-aY}px`,
        width: `${w}px`,
        height: `${h}px`,
        zIndex: zIndex + defaultZIndex + 30,
        opacity: `${t}`,
        lineHeight: `${h > fontSize ? fontSize : h}px`,
        // lineHeight:'100%',

        transformOrigin: `${aX}px ${aY}px`,
        transform: `translate(${x}px, ${y}px) rotate(${r}deg) scaleX(${sX}) scaleY(${sY})`,
        textAlign: textAligns[textAlign],
        fontSize: `${fontSize}px`,
        color: textColor,
        fontFamily: `${font_face ? font_face.post_script_name : "MyFont"}`,
        WebkitTextStroke: `${material.asset.ui.strokeOverFill
            ? strokeWidth + "px" + strokeColor
            : "none"
          }`,
      },
      textBorderStyle = {
        ...textStyle,

        borderWidth: `${2 / Math.abs(sX)}px`,
      };
    //文字处理，如果是下描边，显示dom，另外的元素只显示文字，而下面的元素显示描边
    let newTextStyle = { ...textStyle };
    newTextStyle.zIndex = textStyle.zIndex - 1;
    newTextStyle.color = `${strokeColor}`;
    newTextStyle.fontSize = `${fontSize}px`;
    newTextStyle.WebkitTextStroke = `${strokeWidth}px ${strokeColor}`;
    newTextStyle.display =
      !material.asset.ui.strokeOverFill && strokeWidth && strokeColor
        ? "block"
        : "none";
    return (
      <div className={`${styles.poa}`} key={key} style={containerStyle}>
        <div
          className={`${styles.poa} ${styles.textareaBorderBox}`}
          style={textBorderStyle}
        />
        <textarea
          ref={(ref) => setMaterialRef(ref, key)}
          maxLength={maxLength}
          className={styles.textarea}
          onChange={(e) => handleTextMaterialInput(e, key, maxLength, material)}
          style={textStyle}
          value={defaultText}
          onBlur={handleEditTextBlur.bind(this, maxLength)}
          onDoubleClick={handleTextMaterilaDoubleClick.bind(this, key)}
          onFocus={(e) => handleTextMaterilaFocus(e, key, maxLength, material)}
        />

        {/* 加一层dom进行字体遮挡 */}

        <textarea
          maxLength={maxLength}
          suppressContentEditableWarning
          value={defaultText}
          readOnly={true}
          className={styles.newtextarea}
          style={newTextStyle}
        />
      </div>
    );
  };

  renderAllMaterial = () => {
    const {
      allMaterialByGroup,
      editCurrentGroupKey,
      editMainContainerInfo,
      assetList,
    } = this.props,
      { renderImageAsset, renderTextAsset } = this,
      allMaterial = allMaterialByGroup[editCurrentGroupKey];
    // originAllmaterial = allOriginMaterialByGroup[editCurrentGroupKey];

    if (!allMaterial) return;

    return renderPreviewAsset(
      allMaterial,
      editMainContainerInfo.scale,
      assetList,
      renderImageAsset,
      renderTextAsset,
    );
  };

  getTransformMatrix = (matVar) => {
    var newAffine = new Affine();
    newAffine.set(matVar.anchor, matVar.position, matVar.scale, matVar.rotate);
    return newAffine;
  };

  composeTransformMaxtrix = (anchor, position, scale, rotate) => {
    var newAffine = new Affine();
    newAffine.set(anchor, position, scale, rotate);
    return newAffine;
  };

  mousePressEvent = (e) => {
    const { dispatch } = this.props;
    let imageAsset = this.getCurrentMaterial();
    this.frame.referRotate = imageAsset.rotate;
    this.frame.referScale = {
      x: imageAsset.scale[0],
      y: imageAsset.scale[1],
    };

    this.frame.referPoint = this.getLocalPos(e);

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "setNeedAutoAdapt",
        data: false,
      },
    });
  };

  getContainerMatrix = (imageAsset) => {
    let { p, a, s, r } = imageAsset.asset.ui;
    let containerInfo = this.getContainerInfo();
    let containerVar = {
      anchor: a,
      position: p,
      scale: [s[0] * containerInfo.scale, s[1] * containerInfo.scale],
      rotate: r,
    };
    return MatrixUtil.getTransformMatrix(containerVar);
  };

  getAssetMatrix = (imageAsset) => {
    let { position, anchor, scale, rotate } = imageAsset;

    let matVar = {
      anchor: anchor,
      position: position,
      scale: scale,
      rotate: rotate,
    };
    return MatrixUtil.getTransformMatrix(matVar);
  };

  handleDrag = (e) => {
    let { clientX, clientY } = e;
    let imageAsset = this.getCurrentMaterial();
    const { dispatch } = this.props;

    let local = this.getLocalPos(e);

    let position = MatrixUtil.toVec2(imageAsset.position);
    let offsetX = position.x + local.x - this.frame.referPoint.x;
    let offsetY = position.y + local.y - this.frame.referPoint.y;

    this.frame.referPoint = local;

    // console.log("onDrag", position, local, offsetX, offsetY, this.frame);
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "setPosition",
        data: [offsetX, offsetY],
      },
    });
  };

  handleDragEnd = () => {
    const { dispatch } = this.props;
    // const { dispatch, currentIndex, currentMaterialKey } = this.props;
    // const { materialRefList } = this.state;
    // if (!position.lastEvent) {
    //   position.lastEvent = {
    //     beforeTranslate: [0, 0],
    //   };
    // }
    // const target = materialRefList[currentMaterialKey];
    // const offsetX = position.lastEvent.beforeTranslate[0],
    //   offsetY = position.lastEvent.beforeTranslate[1];
    // dispatch({
    //   type: "workBench/changeGroupData",
    //   payload: {
    //     type: "position",
    //     data: {
    //       offsetX: offsetX,
    //       offsetY: offsetY,
    //     },
    //   },
    // });
    // 隐藏遮罩
    dispatch({
      type: "workBench/save",
      payload: {
        isShowMaterialMaskCanvas: false,
      },
    });
  };

  handleScale = (e) => {
    let { direction } = e;
    let { dispatch } = this.props;
    let imageAsset = this.getCurrentMaterial();
    let { editBoxRefList } = this.state;
    let { position, originSize: imgSize } = imageAsset;

    let scaleInfo = MatrixUtil.getScaleInfo(direction, imgSize);
    let { anchor } = scaleInfo;

    let local = this.getLocalPos(e);
    position = this.getAssetMatrix(imageAsset).getPositionFromAnchor(
      anchor.x,
      anchor.y,
    );

    let { scale } = MatrixUtil.calculateScale(
      scaleInfo,
      local,
      position,
      imageAsset.rotate,
      this.frame.referScale,
    );

    // 初始化一个目标矩阵，锚点和位置都在右下角
    var newAffine = new Affine();
    newAffine.set(anchor, position, scale, imageAsset.rotate);

    position = newAffine.getPositionFromAnchor(
      imageAsset.anchor[0],
      imageAsset.anchor[1],
    );

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "setPosition",
        data: [position.x, position.y],
      },
    });
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "setScale",
        data: [scale.x, scale.y],
      },
    });
  };

  handleScaleEnd = () => {
    const { dispatch } = this.props;
    this.backwindowStatus = false;
    dispatch({
      type: "workBench/save",
      payload: {
        isShowMaterialMaskCanvas: false,
      },
    });
  };

  handleRotate = (e) => {
    let { dispatch } = this.props;
    let imageAsset = this.getCurrentMaterial();
    let { position } = imageAsset;

    let local = this.getLocalPos(e);
    position = MatrixUtil.toVec2(position);

    let start = {
      x: this.frame.referPoint.x - position.x,
      y: this.frame.referPoint.y - position.y,
    };

    let end = {
      x: local.x - position.x,
      y: local.y - position.y,
    };

    let rotate = this.frame.referRotate + Affine.angleBetween(start, end);
    // console.log("rotate", local, position, this.frame, start, end, rotate);

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "setRotate",
        data: rotate,
      },
    });
  };

  handleRotateEnd = () => {
    const { dispatch } = this.props;
    //点击旋转和缩放的时候不触发backwindow事件
    this.backwindowStatus = false;
    // if (!lastEvent) return;
    // this.rotatex = lastEvent.clientX;
    // this.rotatey = lastEvent.clientY;
    // dispatch({
    //   type: "workBench/changeGroupData",
    //   payload: {
    //     type: "rotate",
    //     data: {
    //       rotate: lastEvent.beforeRotate,
    //     },
    //   },
    // });

    dispatch({
      type: "workBench/save",
      payload: {
        isShowMaterialMaskCanvas: false,
      },
    });
  };

  handleTransformStart = (e) => {
    const { dispatch } = this.props;
    this.mousePressEvent(e);
    dispatch({
      type: "workBench/save",
      payload: {
        isShowMaterialMaskCanvas: true,
      },
    });
  };

  renderTransform = () => {
    const { isShowTransform, currentMaterialKey } = this.props;
    if (!isShowTransform) return null;
    const {
      handleDrag,
      handleDragEnd,
      handleScale,
      handleScaleEnd,
      handleRotate,
      handleRotateEnd,
      handleTransformStart,
      getCurrentMaterial,
    } = this,
      currentMaterial = getCurrentMaterial();
    let { materialRefList, bigtarget } = this.state,
      target = materialRefList[currentMaterialKey];

    if (!target) return null;
    let dragTarget = target;
    if (this.multilayerBackground) {
      dragTarget = bigtarget;
    }

    return (
      <Moveable
        target={target}
        bigtarget={dragTarget}
        currentMaterial={{ ...currentMaterial }}
        handleScale={handleScale}
        handleScaleEnd={handleScaleEnd}
        handleDrag={handleDrag}
        handleDragEnd={handleDragEnd}
        handleRotate={handleRotate}
        handleRotateEnd={handleRotateEnd}
        handleScaleStart={handleTransformStart}
        handleDragStart={handleTransformStart}
        handleRotateStart={handleTransformStart}
      />
    );
  };

  handleWatermarkTransformScaleEnd = () => {
    // if (lastEvent) {
    //   const { scale } = lastEvent,
    //     { currentWatermarkKey, watermarkList, dispatch } = this.props,
    //     currentIndex = this.getWatermark(watermarkList, currentWatermarkKey),
    //     currentWatermark = watermarkList[currentIndex];
    //   dispatch({
    //     type: "workBench/changeWatermarkData",
    //     payload: {
    //       type: "setScale",
    //       data: {
    //         scale: [
    //           currentWatermark.scale[0] * scale[0],
    //           currentWatermark.scale[1] * scale[1],
    //         ],
    //       },
    //       currentIndex: currentIndex,
    //     },
    //   });
    // }
  };

  handleWatermarkTransformDragEnd = () => {
    // if (lastEvent) {
    //   const { beforeTranslate } = lastEvent,
    //     { currentWatermarkKey, watermarkList, dispatch } = this.props,
    //     currentIndex = this.getWatermark(watermarkList, currentWatermarkKey),
    //     { position, positionIndex } = watermarkList[currentIndex];
    //   dispatch({
    //     type: "workBench/changeWatermarkData",
    //     payload: {
    //       type: "setPosition",
    //       data: {
    //         activeIndex: positionIndex,
    //       },
    //       currentIndex: currentIndex,
    //     },
    //   });
    // }
  };

  handleWatermarkMousePressEvent = (e) => {
    let imageAsset = this.getCurrentWatermark();
    this.watermarkFrame.referRotate = imageAsset.rotate;
    this.watermarkFrame.referScale = {
      x: imageAsset.scale[0],
      y: imageAsset.scale[1],
    };

    this.watermarkFrame.referPoint = this.getGroupPos(e.clientX, e.clientY);
  };

  handleWatermarkDragStart = (e) => {
    this.handleWatermarkMousePressEvent(e);
  };

  handleWatermarkDrag = (e) => {
    let { currentWatermarkKey, watermarkList, dispatch } = this.props,
      currentIndex = this.getWatermark(watermarkList, currentWatermarkKey),
      { positionIndex } = watermarkList[currentIndex];

    let imageAsset = this.getCurrentWatermark();
    let local = this.getGroupPos(e.clientX, e.clientY);

    let position = MatrixUtil.toVec2(imageAsset.position);
    let offsetX = position.x + local.x - this.watermarkFrame.referPoint.x;
    let offsetY = position.y + local.y - this.watermarkFrame.referPoint.y;

    this.watermarkFrame.referPoint = local;

    dispatch({
      type: "workBench/changeWatermarkData",
      payload: {
        type: "setPosition",
        data: {
          position: [offsetX, offsetY],
          activeIndex: positionIndex,
        },
        currentIndex: currentIndex,
      },
    });
  };

  handleWatermarkScaleStart = (e) => {
    this.handleWatermarkMousePressEvent(e);
  };

  handleWatermarkScale = (e) => {
    let imageAsset = this.getCurrentWatermark();

    let { position, width, height } = imageAsset;
    let { currentWatermarkKey, watermarkList, dispatch } = this.props,
      currentIndex = this.getWatermark(watermarkList, currentWatermarkKey),
      { positionIndex } = watermarkList[currentIndex];

    let imgSize = [width, height];
    let { direction } = e;
    let scaleInfo = MatrixUtil.getScaleInfo(direction, imgSize);
    let { anchor } = scaleInfo;

    let local = this.getGroupPos(e.clientX, e.clientY);
    position = this.getAssetMatrix(imageAsset).getPositionFromAnchor(
      anchor.x,
      anchor.y,
    );

    let { scale } = MatrixUtil.calculateScale(
      scaleInfo,
      local,
      position,
      imageAsset.rotate,
      this.watermarkFrame.referScale,
    );

    // 初始化一个目标矩阵，锚点和位置都在右下角
    var newAffine = new Affine();
    newAffine.set(anchor, position, scale, imageAsset.rotate);

    position = newAffine.getPositionFromAnchor(
      imageAsset.anchor[0],
      imageAsset.anchor[1],
    );

    dispatch({
      type: "workBench/changeWatermarkData",
      payload: {
        type: "setPosition",
        data: {
          position: [position.x, position.y],
          activeIndex: positionIndex,
        },
        currentIndex: currentIndex,
      },
    });
    dispatch({
      type: "workBench/changeWatermarkData",
      payload: {
        type: "setScale",
        data: {
          scale: [scale.x, scale.y],
          activeIndex: positionIndex,
        },
        currentIndex: currentIndex,
      },
    });
  };

  handleWatermarkTransformRotateEnd = () => { };

  renderWatermarkTransform = () => {
    const { isShowWatermarkTransform, currentWatermarkKey } = this.props;
    if (!isShowWatermarkTransform) return null;
    const {
      handleWatermarkDragStart,
      handleWatermarkDrag,
      handleWatermarkScaleStart,
      handleWatermarkScale,
      handleWatermarkTransformScaleEnd,
      handleWatermarkTransformDragEnd,
      getCurrentWatermark,
    } = this,
      currentMaterial = getCurrentWatermark(),
      { watermarkRefList } = this.state;

    return (
      <Moveable
        disableRotatable={true}
        target={watermarkRefList[currentWatermarkKey]}
        currentMaterial={{ ...currentMaterial }}
        handleDragStart={handleWatermarkDragStart}
        handleDrag={handleWatermarkDrag}
        handleScaleStart={handleWatermarkScaleStart}
        handleScale={handleWatermarkScale}
        handleScaleEnd={handleWatermarkTransformScaleEnd}
        handleDragEnd={handleWatermarkTransformDragEnd}
      />
    );
  };

  // renderDragImageList = () => {
  //   return <DragImageList />;
  // };

  handleWatermarkClick = (key, e) => {
    const { dispatch } = this.props;
    e.stopPropagation();
    dispatch({
      type: "workBench/save",
      payload: {
        editWindowCurrentIndex: 4,
        isShowTransform: false,
        isShowWatermarkTransform: true,
        currentWatermarkKey: key,
      },
    });
  };

  renderWatermark = (containerInfo) => {
    const { setWatermarkRef, handleWatermarkClick } = this,
      { watermarkList } = this.props,
      defaultZindex = 100,
      containerStyle = {
        // transform: `scale(${containerScale})`,
      };
    return watermarkList.map((item, i) => {
      const { url, scale, rotate, position, anchor, id, width, height } = item;

      let matrixString = MatrixUtil.composeTransformMatrix(
        anchor,
        position,
        scale,
        rotate,
      ).toMatrixString();
      let stylesObject = {
        transformOrigin: `0px 0px`,
        transform: `${matrixString}`,
        width: width,
        height: height,
      };

      containerStyle.zIndex = defaultZindex + i;

      return (
        <div
          key={id}
          style={containerStyle}
          className={`${styles.watermarkContainer}`}
        >
          <img
            className={`${styles.watermarkItem}`}
            onClick={(e) => handleWatermarkClick(id, e)}
            ref={(ref) => setWatermarkRef(ref, id)}
            style={stylesObject}
            src={url}
            alt="watermarkImage"
          />
        </div>
      );
    });
  };



  renderMaskCanvas = (containerInfo) => {
    const { width, height } = containerInfo,
      { isShowMaterialMaskCanvas } = this.props;
    return (
      <canvas
        ref={(ref) => this.setState({ maskCanvasDom: ref })}
        style={{ display: isShowMaterialMaskCanvas ? 'block' : 'none' }}
        className={`${styles.renderMaterialMask}`}
        width={width}
        height={height}
      />
    );
  };
  renderLoading = (status, text) => {
    let load = this.props.editMainLoading;
    //可能多次出现消失，用类名控制性能更好
    return (
      <div
        className={`${styles.loadingEditorContainer} ${!load ? styles.hideloading : ""
          }`}
      >
        <div className={`flex ${styles.loadingBackground}`}>
          <Spin size="large" tip={`${text}...`} />
        </div>
      </div>
    );
  };
  renderCanvasMask = (containerInfo) => {
    const {
      allOriginMaterialByGroup,
      editCurrentGroupKey,
      currentMaterialKey,
    } = this.props,
      { width, height } = containerInfo,
      { maskCanvasDom } = this.state;
    if (!maskCanvasDom) return;
    // if (!isShowMaterialMaskCanvas) return;
    if (!editCurrentGroupKey) return;
    if (!currentMaterialKey) return;

    const context = maskCanvasDom.getContext("2d"),
      currentMaterialList = allOriginMaterialByGroup[editCurrentGroupKey],
      currentMaterialIndex = getIndexByAssetName(
        currentMaterialKey,
        currentMaterialList,
      ),
      currentMaterial = currentMaterialList[currentMaterialIndex];
    // s = currentMaterial.scale[0],
    // x = currentMaterial.position[0],
    // y = currentMaterial.position[1],
    // aX = currentMaterial.anchor[0],
    // aY = currentMaterial.anchor[1],
    // w = currentMaterial.originSize[0],
    // h = currentMaterial.originSize[1],
    // r = currentMaterial.rotate,
    // matrix = new Affine(),
    // anchor = { x: aX, y: aY },
    // position = { x: x, y: y },
    // zoom = { x: s, y: s };

    let imageAsset = currentMaterial;
    let { p, a, s, r, editSize } = imageAsset.asset.ui;

    let initMatVar = {
      anchor: a,
      position: p,
      scale: s,
      rotate: r,
    };
    context.save();
    // matrix.set(anchor, position, zoom, r);

    let matrix = MatrixUtil.getTransformMatrix(initMatVar);

    context.transformOrigin = "0px 0px";
    context.clearRect(0, 0, width, height);
    context.fillStyle = "rgba(0,0,0, 0.7)";
    context.fillRect(0, 0, width, height);
    context.globalCompositeOperation = "destination-out";
    context.setTransform(
      matrix.mA,
      matrix.mB,
      matrix.mC,
      matrix.mD,
      matrix.mE,
      matrix.mF,
    );
    context.fillStyle = "rgba(255,255,255)";
    context.fillRect(0, 0, editSize[0], editSize[1]);
    context.resetTransform();
    context.restore();
  };

  componentDidMount() {
    const { dispatch } = this.props;

    sessionStorage.setItem("checkAutoSize", "");
    //进入页面字体加载默认显示1.2s的loading。
    setTimeout(() => {
      dispatch({
        type: "workBench/save",
        payload: {
          editMainLoading: false,
        },
      });
    }, 1200);

  }

  componentWillUnmount() {
    const { dispatch } = this.props;

    dispatch({
      type: "workBench/save",
      payload: {
        editMainLoading: true,
      },
    });
  }
  backwindow = () => {
    const { dispatch } = this.props;
    const { materialRefList } = this.state;
    if (!this.backwindowStatus) {
      this.backwindowStatus = true;
      return;
    }
    for (var i in materialRefList) {
      materialRefList[i].blur();
    }
    if (this.props.editWindowCurrentIndex !== 1) {
      dispatch({
        type: "workBench/save",
        payload: {
          editWindowCurrentIndex: 1,
          currentMaterialKey: "",
          isShowTransform: false,
          currentWatermarkKey: "", //点击空白返回全局，当前的水印选中状态也清空
        },
      });
    }
  };

  compareAssetIndex = (obj1, obj2) => {
    var val1 = obj1.zIndex;
    var val2 = obj2.zIndex;
    if (val1 < val2) {
      return 1;
    } else if (val1 > val2) {
      return -1;
    } else {
      return 0;
    }
  };

  //根据滑过坐标的位置确定选中的图片assetName
  getCordPictureKey = (e) => {
    if (
      e.target.className.indexOf("poa") !== -1 ||
      e.target.className.indexOf("textarea") !== -1
    ) {
      return null;
    }
    let nowImageKey = null,
      x = 0,
      y = 0;
    const { allMaterialByGroup, editCurrentGroupKey } = this.props,
      { scale } = this.getContainerInfo(),
      nowGroup = allMaterialByGroup[editCurrentGroupKey];

    let local = this.getGlobalPos(e.clientX, e.clientY);

    let { groupContainerRefList } = this.state;
    let groupBox = groupContainerRefList[editCurrentGroupKey];
    let groupBoxBound = groupBox.getBoundingClientRect();

    local.x -= groupBoxBound.x;
    local.y -= groupBoxBound.y;
    //模板上当前的真实坐标
    x = local.x;
    y = local.y;

    // 获取按照 index 顺序排序后的素材
    let sortAssets = [];
    let count = 0;
    nowGroup.forEach((v) => {
      let item = { zIndex: v.zIndex, index: count };
      count++;
      sortAssets.push(item);
    });

    // 排序，zIndex 越小越后
    sortAssets.sort(this.compareAssetIndex);

    //根据clickArea点击区域确定当前点击的图片
    for (let i = 0; i < sortAssets.length; i++) {
      let v = nowGroup[sortAssets[i].index];
      if (!v.fontSetting) {
        let area = {
          x0: v.clickArea[0] * scale,
          x1: v.clickArea[2] * scale + v.clickArea[0] * scale,
          y0: v.clickArea[1] * scale,
          y1: v.clickArea[3] * scale + v.clickArea[1] * scale,
        };

        // console.log("select ", x, y, local, area);
        if (x >= area.x0 && x <= area.x1 && y >= area.y0 && y <= area.y1) {
          nowImageKey = v.assetName;
          break;
        }
      }
    }

    return nowImageKey;
  };
  eventPenetration = (e, type) => {
    let nowImageKey = this.getCordPictureKey(e),
      { materialRefList } = this.state;
    if (nowImageKey) {
      let imageIndex = materialRefList[nowImageKey].parentNode.style.zIndex;
      this.imageIndex = imageIndex;
      if (type === "dbclick") {
        this.handleImageMaterialDoubleClick(
          materialRefList[nowImageKey],
          nowImageKey,
        );
      } else {
        //是否改变拖动元素
        this.multilayerBackground = true;
        materialRefList[nowImageKey].click();
      }
    }
  };
  stopparent = (e) => {
    e.stopPropagation();
    this.eventPenetration(e);
  };
  setbigtarget = (ref) => {
    this.setState({
      bigtarget: ref,
    });
  };

  render() {
    let {
      getContainerInfo,
      getContainerStyle,
      renderAccessory,
      getBackgroundAccessorySrc,
      getProspectAccessorySrc,
      renderAllMaterial,
      renderTransform,
      // renderDragImageList,d
      handleEditContainerMouseUp,
      renderMaskCanvas,
      renderCanvasMask,
      renderWatermark,
      renderLoading,
      renderWatermarkTransform,
      setGroupContainerRef,
    } = this,
      { currentImageAssetChange, fileIdCount } = this.state,
      { isFullEditMain, editCurrentGroupKey, parseManage } = this.props,
      containerInfo = getContainerInfo(),
      containerStyle = getContainerStyle(containerInfo);
    const fullClass = isFullEditMain ? styles.activeFull : "";
    renderCanvasMask(containerInfo);
    return (
      <>
        <div
          ref={(ref) => this.setbigtarget(ref)}
          className={`flex ${styles.editMainContainer}`}
        >
          <div
            onClick={this.backwindow}
            className={`flex ${styles.editorLayoutContainer} ${fullClass}`}
          >
            <div
              ref={(ref) => setGroupContainerRef(ref, editCurrentGroupKey)}
              style={containerStyle}
              className={styles.editorLayout}
              onClick={(e) => this.stopparent(e)}
              // onDoubleClick={(e) => this.eventPenetration(e, "dbclick")}
              onMouseUp={handleEditContainerMouseUp}
            // onDrag={(e) => this.eventPenetration(e, 'drag')}
            >
              {renderAccessory()}
              {/* {renderBeforeImage()} */}
              {renderAllMaterial()}
              {renderWatermark(containerInfo)}
              {renderMaskCanvas(containerInfo)}
              {renderLoading("loading", "加载中...")}
            </div>

            {renderTransform()}
            {renderWatermarkTransform()}

            {/* <div
              style={containerStyle}
              className={`${styles.editorLayoutEmpty}`}
            /> */}
          </div>
        </div>
        {/* 拖拽列表 */}
        {/* {renderDragImageList()} */}
        <input
          id={`fileUpload_${fileIdCount}`}
          type="file"
          accept="image/*,video/*"
          style={{ display: "none" }}
          ref={(ref) => this.setState({ uploadFileRef: ref })}
          onChange={currentImageAssetChange}
        />
      </>
    );
  }
}

const stateToProps = (state) => {
  const { workBench } = state,
    {
      editMainContainerInfo,
      parseManage,
      editCurrentGroupKey,
      editMainPdding,
      dragImageMoveing,
      backgroundImageList,
      prospectImageList,
      assetList,
      isShowTransform,
      currentMaterialKey,
      currentSelectImageList,
      allMaterialByGroup,
      allOriginMaterialByGroup,
      isShowMaterialMaskCanvas,
      currentWatermarkKey,
      isShowWatermarkTransform,
      watermarkList,
      isFullEditMain,
      editMainLoading,
      origin_draft,
      mattingObj,
      selectMaterialType,
      cut_origin_draft,
    } = workBench;

  return {
    editMainLoading: editMainLoading,
    origin_draft,
    mattingObj,
    selectMaterialType,
    cut_origin_draft,
    parseManage: parseManage,
    assetList: assetList, //当前模板所有的图片
    editMainPdding: editMainPdding,
    editMainContainerInfo: editMainContainerInfo,
    editCurrentGroupKey: editCurrentGroupKey,
    backgroundImageList: backgroundImageList,
    prospectImageList: prospectImageList,
    currentMaterialKey: currentMaterialKey,
    allMaterialByGroup: allMaterialByGroup,
    isShowTransform: isShowTransform,
    dragImageMoveing: dragImageMoveing,
    currentSelectImageList: currentSelectImageList,
    allOriginMaterialByGroup: allOriginMaterialByGroup,
    isShowMaterialMaskCanvas: isShowMaterialMaskCanvas,
    currentWatermarkKey: currentWatermarkKey,
    isShowWatermarkTransform: isShowWatermarkTransform,
    watermarkList: watermarkList,
    isFullEditMain: isFullEditMain,
  };
};

export default connect(stateToProps)(EditMain);
