import React, { PureComponent } from "react";
import { connect } from "dva";
import styles from "./index.less";
import Scrollbars from "react-custom-scrollbars";
import { getIndexByAssetName } from "utils/lib/mapUtils";
import textColorList from "utils/textColorList";
import { ChromePicker } from "react-color";
import chromaticCircle from "@/assets/workBench/chromaticCircle.svg";
import { Button, message, Slider, Checkbox, Tooltip } from "antd";
import app from "@/utils/lib/app";
class EditFont extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      isShowColorSelect: false,
      showPreInstallColorIndex: -1,
      colorSelectIndex: -1,
      openAutosize: true,
    };
  }

  closeSelectColor = () => {
    this.setPreInstallColorListStatus(-1);
    this.setIsShowColorSelectStatus(false, -1);
  };

  componentDidMount() {
    this.setState({
      openAutosize:
        sessionStorage.getItem("checkAutoSize") === "closeAutosize"
          ? false
          : true,
    });

    document.addEventListener("click", this.closeSelectColor);
  }

  componentWillUnmount() {
    document.removeEventListener("click", this.closeSelectColor);
  }

  setPreInstallColorListStatus = (type) => {
    if (type === this.state.showPreInstallColorIndex) {
      this.setState({
        showPreInstallColorIndex: -1,
      });
      return;
    }
    this.setState({
      showPreInstallColorIndex: type,
    });
  };

  setIsShowColorSelectStatus = (status, type) => {
    this.setState({
      isShowColorSelect: status,
      colorSelectIndex: type,
    });
  };

  openPreInstallColorList = (e, type) => {
    e.nativeEvent.stopImmediatePropagation();
    this.setPreInstallColorListStatus(type);
  };

  handleWrapClick = (e) => {
    e.nativeEvent.stopImmediatePropagation();
  };

  getCurrentMaterial = () => {
    const {
        allMaterialByGroup,
        editCurrentGroupKey,
        currentMaterialKey,
      } = this.props,
      allMaterialList = allMaterialByGroup[editCurrentGroupKey],
      index = getIndexByAssetName(currentMaterialKey, allMaterialList);

    return allMaterialList[index];
  };

  changeColor = (val, type) => {
    const { dispatch } = this.props;
    let actionType = "changeTextColor";
    if (type === 2) actionType = "changeStrokeColor";

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: actionType,
        data: {
          color: val,
        },
      },
    });
  };

  renderColorList = (type) => {
    return textColorList.map((colorItem) => {
      const { id, color } = colorItem;

      return (
        <div
          onClick={this.changeColor.bind(this, color, type)}
          key={id}
          style={{ backgroundColor: `${color}` }}
          className={`cursorPointer ${styles.preinstallColorItem}`}
        />
      );
    });
  };

  handleEditTextChange = (maxLength, e) => {
    const { dispatch, equalList } = this.props,
      val = e.target.value;

    //文字自适应
    let tempobj = this.getCurrentMaterial();
    let size = app.getAutoSize(tempobj, val);
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: val,
          size,
        },
      },
    });
    if (equalList.length) {
      dispatch({
        type: "workBench/changeEqualText",
        payload: {
          data: {
            text: val,
            equlist: equalList,
            size,
          },
        },
      });
    }
  };

  handleEditTextBlur = (maxLength, e) => {
    const { dispatch } = this.props;
    let val = e.target.value;
    if (val.length > maxLength) {
      message.info("文字字数超过模板的最大限制");
      val = val.substring(0, maxLength);
    }

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: val,
        },
      },
    });
  };

  renderEditTextContainer = (fontData) => {
    if (!fontData) {
      return false;
    }

    const { defaultText, maxLength } = fontData;
    return (
      <div className={`${styles.editTextContainer}`}>
        <textarea
          value={defaultText}
          maxLength={maxLength}
          onChange={this.handleEditTextChange.bind(this, maxLength)}
          onBlur={this.handleEditTextBlur.bind(this, maxLength)}
        />
        <div className={`${styles.textCountInfo}`}>
          <span className={`${styles.currentTextCount}`}>
            {defaultText ? defaultText.length : 0}
          </span>
          /{maxLength}
        </div>
      </div>
    );
  };

  handleEditFontSizeChange = (e) => {
    let val = e.target.value;
    val = val.replace(/\D/g, "");
    this.handleSetFontSize(val);
  };

  handleSetFontSize = (val) => {
    const { dispatch, equalList } = this.props,
      { fontSetting } = this.getCurrentMaterial(),
      { defaultText } = fontSetting;

    if (this.state.openAutosize) {
      this.setState({
        openAutosize: false,
      });
      sessionStorage.setItem("checkAutoSize", "closeAutosize");
    }
    if (equalList.length) {
      dispatch({
        type: "workBench/changeEqualText",
        payload: {
          data: {
            text: defaultText,
            equlist: equalList,
            size: val,
          },
        },
      });
    }
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeFontSize",
        data: {
          size: val,
          text: defaultText,
        },
      },
    });
  };

  renderEditFontSize = (fontData) => {
    if (!fontData) {
      return;
    }
    const { fontSize } = fontData;
    return (
      <div className={`flex ${styles.fontSizeContainer}`}>
        <div className={`flex borderBox ${styles.fontSizeInput}`}>
          <input value={fontSize} onInput={this.handleEditFontSizeChange} />
          <span>px</span>
        </div>
        <div className={`flex borderBox ${styles.fontSizeBtn}`}>
          <button
            onClick={() =>
              this.handleSetFontSize(Number(fontSize) + 1, fontData)
            }
            className={`borderBox cursorPointer`}
          >
            A+
          </button>
          <button
            onClick={() =>
              this.handleSetFontSize(Number(fontSize) - 1, fontData)
            }
            className={`borderBox cursorPointer`}
          >
            A-
          </button>
        </div>
      </div>
    );
  };

  renderPreInstallColorList = (type) => {
    const { showPreInstallColorIndex } = this.state;

    if (showPreInstallColorIndex !== type) return null;

    return (
      <div
        onClick={this.handleWrapClick}
        className={`flex ${styles.preinstallColorContainer}`}
      >
        <div className={`${styles.preinstallColorTitle}`}>预设颜色</div>
      
        <Scrollbars>
          <div className={`flex ${styles.preinstallColorList}`}>
            {this.renderColorList(type)}
          </div>
        </Scrollbars>
      </div>
    );
  };

  openColorSelect = (e, type) => {
    this.handleWrapClick(e);
    this.setIsShowColorSelectStatus(true, type);
  };

  renderEditColor = (type, fontData) => {
    if (!fontData) {
      return;
    }
    const { strokeColor, textColor } = fontData;
    let color = textColor;
    if (type === 2) color = strokeColor;
    return (
      <div className={`flex ${styles.editColorContainer}`}>
        <div
          onClick={(e) => this.openColorSelect(e, type)}
          style={{ backgroundColor: `${color}` }}
          className={`cursorPointer ${styles.colorPreview}`}
        />
        <div
          onClick={(e) => this.openPreInstallColorList(e, type)}
          style={{ backgroundImage: `url(${chromaticCircle})` }}
          className={`cursorPointer ${styles.colorSelectAction}`}
        />
        {this.renderPreInstallColorList(type, fontData)}
      </div>
    );
  };

  selectColorChange = (e, type) => {
    const colorHex = e.hex;
    this.changeColor(colorHex, type);
  };

  renderSelectColorContainer = (fontData) => {
    if (!fontData) {
      return false;
    }
    const { isShowColorSelect, colorSelectIndex } = this.state,
      { strokeColor, textColor } = fontData;
    if (!isShowColorSelect) return false;
    let color = textColor;
    if (colorSelectIndex === 2) color = strokeColor;

    return (
      <div
        onClick={this.handleWrapClick}
        className={`${styles.colorSelectBox}`}
      >
        <ChromePicker
          disableAlpha={true}
          color={color}
          onChange={(e) => this.selectColorChange(e, colorSelectIndex)}
        />
      </div>
    );
  };
  //改变自适应状态
  onRadioChange = (e) => {
    const checkAutoSize =
      sessionStorage.getItem("checkAutoSize") === "openAutoSize"
        ? "closeAutosize"
        : "openAutoSize";
    // const { checkAutoSize, dispatch } = this.props;
    sessionStorage.setItem("checkAutoSize", checkAutoSize);
    this.setState({
      openAutosize: checkAutoSize === "closeAutosize" ? false : true,
    });
    // dispatch({
    //   type: "workBench/save",
    //   payload: {
    //     checkAutoSize: !checkAutoSize,
    //   },
    // });
  };
  changeWidth = (val) => {
    const { dispatch } = this.props;
    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeStrokeWidth",
        data: {
          width: val,
        },
      },
    });
  };

  renderStrokeWidthSlider = (fontData) => {
    if (!fontData) {
      return;
    }
    const { strokeWidth } = fontData;

    return (
      <div className={`${styles.slider}`}>
        <Slider value={strokeWidth} max={10} onChange={this.changeWidth} />
      </div>
    );
  };
  renderCheckAuto = () => {
    const { openAutosize } = this.state;

    const tooltipInfo = (
      <span style={{ color: "#888", fontSize: 12 }}>
        视频生成后文字最终大小可能与当前效果不一致，超出文字框外的文字会自适应大小到文本框内。
      </span>
    );
    return (
      <div className={styles.radio_tip}>
        <Checkbox onClick={()=>this.onRadioChange()} checked={openAutosize}>
          <span className={styles.radio_text}>文字大小自适应</span>
        </Checkbox>

        <Tooltip
          placement="topRight"
          className={styles.tip}
          color="#fff"
          title={tooltipInfo}
        >
        <img className={styles.question_icon} src={require('@/assets/img/question_icon.svg')} alt="" />
      {/* <QuestionCircleOutlined /> */}
          {/* <QuestionCircleOutlined className={styles.question_icon} /> */}
        </Tooltip>
      </div>
    );
  };

  resetAll = () => {
    const {
        editCurrentGroupKey,
        currentMaterialKey,
        allOriginMaterialByGroup,
        dispatch,
        allMaterialByGroup,
      } = this.props,
      allMaterialList = allOriginMaterialByGroup[editCurrentGroupKey],
      index = getIndexByAssetName(currentMaterialKey, allMaterialList);
    let fontSetting = allMaterialList[index].fontSetting;
   
    fontSetting.font_face =
      allMaterialByGroup[editCurrentGroupKey][index].fontSetting.font_face;

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "resetAllFontSetting",
        data: {
          fontSetting: JSON.parse(JSON.stringify(fontSetting)),
        },
      },
    });
  };

  renderResetAll = () => {
    const resetAllStyle = {
      width: "88px",
      height: "24px",
      fontSize: "14px",
      padding: 0,
    };

    return (
      <div className={`flex ${styles.resetAllButtonContainer}`}>
        <Button style={resetAllStyle} onClick={this.resetAll}>
          全部重置
        </Button>
      </div>
    );
  };

  render() {
    const {
        getCurrentMaterial,
        renderEditTextContainer,
        renderStrokeWidthSlider,
        renderEditFontSize,
        renderEditColor,
        renderSelectColorContainer,
        renderCheckAuto,
        renderResetAll,
      } = this,
      { fontSetting } = getCurrentMaterial();

    return (

      <div>
   {renderSelectColorContainer(fontSetting)}
        <div className={`${styles.editFontContainer}`}>
          <div className={`${styles.textLable}`}>文本内容</div>
          {renderEditTextContainer(fontSetting)}
          <div className={`${styles.textLable}`}>文字大小</div>
          {renderCheckAuto()}
          {renderEditFontSize(fontSetting)}
          <div className={`${styles.textLable}`}>文字颜色</div>
          {renderEditColor(1, fontSetting)}
          <div className={`${styles.textLable}`}>描边颜色</div>
          {renderEditColor(2, fontSetting)}
          <div className={`${styles.textLable}`}>描边大小</div>
          {renderStrokeWidthSlider(fontSetting)}
          {renderResetAll()}
        </div>
        
      </div>
     
    );
  }
}

const stateToProps = (state) => {
  const { workBench } = state,
    {
      editCurrentGroupKey,
      allMaterialByGroup,
      currentMaterialKey,
      allOriginMaterialByGroup,
      equalList,
      checkAutoSize, //是否设置字体自适应
    } = workBench;

  return {
    editCurrentGroupKey: editCurrentGroupKey,
    allMaterialByGroup: allMaterialByGroup,
    equalList,
    checkAutoSize,
    currentMaterialKey: currentMaterialKey,
    allOriginMaterialByGroup: allOriginMaterialByGroup,
  };
};
export default connect(stateToProps)(EditFont);

