import React, { PureComponent } from "react";
import { Slider } from "antd";
import { connect } from "dva";
import styles from "./index.less";
import ClipImage from "components/ClipImage";
import app from '@/utils/lib/app'
import Upload from "@/utils/lib/Upload";
import env from "@/env";
import Cookies from "js-cookie";
import { TOKEN_NAME } from "@/types";
import { isVideo, wipeOffQueryString} from "@/utils/index";
class EditGraphic extends PureComponent {

  constructor(props) {
    super(props);
    this.updateId = -1;
    this.state={
      isShowClipImageModel:false,
    }
  }

  getImageUrl() {
    const {assetList,currentMaterialKey,allMaterialByGroup,editCurrentGroupKey} = this.props;
    let imageUrl = assetList[currentMaterialKey] || ''
    let groupArr  = allMaterialByGroup[editCurrentGroupKey];
  let material = {}
    if(groupArr.length){
      groupArr.forEach(v=>{
        if(v.assetName === currentMaterialKey ){
          material = v
        }
      })
    }
    // const { currentGroup, currentMaterialIndex, assetList } = this.props["editor"];
    // for (const key in currentGroup) {
    //   if (currentGroup.hasOwnProperty(key)) {
    //     let list = currentGroup[key],
    //       i = 0,
    //       len = list.length;
    //     for (; i < len; i++) {
    //       let material = list[i];
    //       if (material["key"] === currentMaterialIndex) {
    //         return {
    //           imageUrl: assetList[material["assetName"]],
    //           material: material,
    //         };
    //       }

    //     }
    //   }
    // }

    return {
      imageUrl,
      material,
    };

  }

  changeVolume = (e, material) => {
    clearTimeout(this.updateId);
    this.updateId = setTimeout(() => {
      const { dispatch } = this.props;
      clearTimeout(this.updateId);
      dispatch({
        type: "editor/changeGroupData",
        payload: {
          type: "changeVolume",
          data: {
            volume: e,
          },
        },
      });
    }, 100);
  };

  renderVolumeSlider(isShow, material) {
    if (!isShow) return null;
    const volume = material.volume || 100;

    return (
      <>
        {/* <p className={`borderBox ${styles.groupTitle}`}>上传视频控制</p> */}
        <div className={`flex`}>
        <button onClick={this.showMaterialModel} className={`flex cursorPointer ${styles.button}`}>
          <span className={`iconfont ${styles.uploadIcon}`}>&#xe697;</span>
          更换图片/视频
        </button>
      </div>
        <div style={{ height: 28 }}/>
        <div className={`flex ${styles.volume}`}>
          <span className={`iconfont ${styles.volumeIcon}`}>&#xe693;</span>
          <div className={` ${styles.volumeSlider}`}>
            <Slider defaultValue={volume} max={100} onChange={(e) => this.changeVolume(e, material)}/>
          </div>
        </div>
        <div style={{ height: 32 }}/>
        <div className={styles.editGraphicGroupLine}/>
        <div style={{ height: 24 }}/>
        <div className={`flex`}>
          <button onClick={this.showVideoClipBox} className={`flex cursorPointer ${styles.button}`}>
            <span className={`iconfont ${styles.uploadIcon}`}>&#xe697;</span>
            裁切视频
          </button>
        </div>
      </>
    );
  }

  renderCorpImageBox(isShow) {
    if (isShow) {
      return null;
    }

    return (
      <>
       <div className={`flex`}>
        <button onClick={this.showMaterialModel} className={`flex cursorPointer ${styles.button}`}>
          <span className={`iconfont ${styles.uploadIcon}`}>&#xe697;</span>
          更换图片/视频
        </button>
      </div>
        {/* <p className={`borderBox ${styles.groupTitle}`}>上传图片控制</p> */}
        <div className={`flex`}>
          <button onClick={this.showCorpBox} className={`flex cursorPointer ${styles.button}`}>
            <span className={`iconfont ${styles.uploadIcon}`}>&#xe697;</span>
            裁切图片
          </button>
        </div>
      </>
    );
  }

  showMaterialModel = () => {
    const { dispatch } = this.props;

    dispatch({
      type: "editor/save",
      payload: {
        isShowMaterialListModel: true,
      },
    });
  };

  showCorpBox = () => {
    const { dispatch } = this.props;
    const { imageUrl } = this.getCurrentOriginMaterial();

    dispatch({
      type: "workBench/save",
      payload: {
        cropPicture: imageUrl,
      },
    });
    this.setState({
      isShowClipImageModel: true,
    });
    // const { dispatch } = this.props;
    // dispatch({
    //   type: "editor/save",
    //   payload: {
    //     isShowCorpImage: true,
    //   },
    // });
  };

  showVideoClipBox = () => {
    const { dispatch } = this.props;
    dispatch({
      type: "editor/save",
      payload: {
        isShowVideoClip: true,
      },
    });
  };
  getCurrentOriginMaterial=()=>{
    const {allMaterialByGroup,currentMaterialKey,assetList} = this.props;
    const  material = app.getMaterial(allMaterialByGroup,currentMaterialKey);
    return{
      material,
      imageUrl:assetList[currentMaterialKey]
    }

  };
  setClipImageModelStatus = (status) => {
    const { dispatch } = this.props;
    const { imageUrl } = this.getCurrentOriginMaterial();

    dispatch({
      type: "workBench/save",
      payload: {
        cropPicture: imageUrl,
      },
    });
    this.setState({
      isShowClipImageModel: status,
    });
  };
  
  handleClipImageOnOk = async (file, noSaveBeforeImage) => {

    const {     dispatch} = this.props;
    const {
 
      assetList,
      editCurrentGroupKey,
      allMaterialByGroup,
      currentMaterialKey,
      allOriginMaterialByGroup,
      origin_draft,
      cut_origin_draft,
    } = this.props['workBench'];
    let tempobj = Object.assign({}, cut_origin_draft);
    dispatch({
      type: "workBench/save",
      payload: {
        editMainLoading: true,
      },
    });
    let upload = new Upload({
      getTokenUrl: env.getTokenUrl,
      jwt: Cookies.get(TOKEN_NAME),
      sourceHost: env.sourceHost,
    });

  let    cropImageUrl = await upload.uploadFile(file),
      isVideoFile = isVideo(wipeOffQueryString(cropImageUrl));
    if (isVideoFile) {
      cropImageUrl += "?x-oss-process=video/snapshot,t_0";
    }

    if (origin_draft) {
      origin_draft[currentMaterialKey] = cropImageUrl;
    }
    //替换图片上传图片的时候需要存原图，因为裁切时点击确定同样掉这个方法，因此加以判断。只让上传的时候存储
    if (tempobj && !noSaveBeforeImage) {
      tempobj[currentMaterialKey] = cropImageUrl;
    }

    dispatch({
      type: "workBench/replaceImage",
      payload: {
        assetName: currentMaterialKey,
        origin_draft: origin_draft,
        assetList: assetList,
        cut_origin_draft: tempobj,
        editCurrentGroupKey: editCurrentGroupKey,
        allMaterialByGroup: allMaterialByGroup,
        allOriginMaterialByGroup: allOriginMaterialByGroup,
        dataItem: {
          imageSrc: cropImageUrl,
        },
      },
    });
  };
  renderClipImageModel = () => {
    const { material } = this.getCurrentOriginMaterial(),
      { isShowClipImageModel } = this.state;
    if (!isShowClipImageModel) return null;

    return (
      <ClipImage
        material={material}
        onOk={this.handleClipImageOnOk}
        setClipImageModelStatus={this.setClipImageModelStatus}
      />
    );
  };

  resetTransform = () => {
    const { dispatch,allMaterialByGroup,currentMaterialKey } = this.props;
    dispatch({
      type:'workBench/initAddress',
      payload:{
        allMaterialByGroup,
        currentMaterialKey,
      }
    })
    // dispatch({
    //   type: "editor/changeGroupData",
    //   payload: {
    //     type: "reset",
    //   },
    // });

    // dispatch({
    //   type: "editor/resetCurrentMaterialTransform",
    //   payload: {
    //     allMaterialByGroup: editor.allMaterialByGroup,
    //     allOriginMaterialByGroup: editor.allOriginMaterialByGroup,
    //     currentMaterialIndex: editor.currentMaterialIndex,
    //     currentGroupIndex: editor.currentGroupIndex,
    //   },
    // });
  };

  renderLocalUploadFile = () => {
    const { editor } = this.props;
    if (editor.isVeLeapTest) return null;
    return (
      <div className={`flex`}>
        <button onClick={this.showMaterialModel} className={`flex cursorPointer ${styles.button}`}>
          <span className={`iconfont ${styles.uploadIcon}`}>&#xe697;</span>
          更换图片/视频
        </button>
      </div>
    );
  };


  render() {
    const { material, imageUrl } = this.getImageUrl();
     const {renderClipImageModel} = this;
    return (
      <div className={styles.editGraphicContainer}>
        <div className={`flex borderBox ${styles.imageBox}`}>
          <img src={imageUrl}  alt="asset"/>
        </div>
        <div className={styles.editGraphicGroupLine}/>
        {this.renderVolumeSlider(material["isVideo"], material)}
        {this.renderCorpImageBox(material["isVideo"])}
        <div className={`flex`}>
          <button onClick={this.resetTransform} className={`flex cursorPointer ${styles.button}`}>
            重置位置
          </button>
        </div>
        {/* 裁切图片 */}
        {renderClipImageModel()}
      </div>
    );
  }
}


const stateToProps = (state) => {
  return {
    editor: state["editor"],
    loading: state["loading"],
    assetList:state['workBench'].assetList,
    workBench:state['workBench'],
    currentMaterialKey:state['workBench'].currentMaterialKey,
    allMaterialByGroup:state['workBench'].allMaterialByGroup,
    editCurrentGroupKey:state['workBench'].editCurrentGroupKey,
    
  };
};

export default connect(stateToProps)(EditGraphic);
