import React, { PureComponent } from "react";
import { connect } from "dva";
import styles from "./index.less";
import Scrollbars from "react-custom-scrollbars";
import Upload from "utils/lib/Upload";
import env from "@/env";
import Cookie from "js-cookie";
import { TOKEN_NAME } from "@/types";
import { isVideo, wipeOffQueryString } from "utils/index";
import { message } from "antd";
import app from "@/utils/lib/app";
class EditGlobal extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      scrollbarRef: null,
      domRefList: {},
      textRefList: {},
      textAreaShowStateList: {},
    };
    this.textList = []; //内容相同的key数组
    this.equlist = []; //点击文案拿到相同数据的key
    this.commonInputStyle = {
      display: "none",
    };
  }

  setDomRef = (ref, key) => {
    this.setState((state) => {
      state.domRefList[key] = ref;
      return {
        ...state,
      };
    });
  };

  setTextRef = (ref, key) => {
    this.setState((state) => {
      state.textRefList[key] = ref;
      return {
        ...state,
      };
    });
  };

  handleTextClick = (assetName, templateobj) => {
    const { textAreaShowStateList, domRefList, textRefList } = this.state;
    //相同的另外的key
    let {  asset } = templateobj;
    this.equlist = [];
    this.textList.forEach((v) => {
      if (
        asset.ui &&
        asset.ui.link &&
        asset.ui.link.length &&
        asset.ui.link.indexOf(v.asset.ui.index) !== -1 &&
        assetName !== v.assetName //避免非相同字相同框内字体自适应
      ) {
        this.equlist.push(v.assetName);
      }
    });

    if (!textAreaShowStateList[assetName]) {
      if (textRefList[assetName] && domRefList[assetName]) {
        textRefList[assetName].style.height =
          domRefList[assetName].offsetHeight + "px";
      }
      this.setState(
        {
          textAreaShowStateList: {
            [assetName]: true,
          },
        },
        () => {
          textRefList[assetName].select();
        },
      );
    }
  };

  handleTextInput = (e, maxLength, key, obj) => {
    const value = e.target.value,
      { dispatch } = this.props;
    const size = app.getAutoSize(obj, value);

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: value,
          size: size,
        },
      },
    });
    //modal中循环替换
    if (this.equlist.length) {
      dispatch({
        type: "workBench/changeEqualText",
        payload: {
          data: {
            text: value,
            equlist: this.equlist,
            size: size,
          },
        },
      });
    }
  };

  handleEditTextBlur = (maxLength, e) => {
    const { dispatch } = this.props;
    let val = e.target.value;
    if (val.length > maxLength) {
      message.info("文字字数超过模板的最大限制");
      val = val.substring(0, maxLength);
    }

    dispatch({
      type: "workBench/changeGroupData",
      payload: {
        type: "changeText",
        data: {
          text: val,
        },
      },
    });
  };

  handleTextBlur = (assetName, e, maxLength) => {
    if (this.state.textAreaShowStateList[assetName]) {
      this.setState({
        textAreaShowStateList: {
          [assetName]: false,
        },
      });
    }
    this.handleEditTextBlur(maxLength, e);
  };

  handleTextFocus = (key) => {
    const { currentMaterialKey, dispatch } = this.props;
    if (currentMaterialKey === key) return;
    dispatch({
      type: "workBench/save",
      payload: {
        currentMaterialKey: key,
        isShowTransform: false,
        currentWatermarkKey: "",
        isShowWatermarkTransform: false,
      },
    });
  };

  handleEditImageClick = (assetName) => {
    const { dispatch } = this.props;
    dispatch({
      type: "workBench/save",
      payload: {
        editWindowCurrentIndex: 2,
        currentMaterialKey: assetName,
        isShowTransform: true,
        currentWatermarkKey: "",
        isShowWatermarkTransform: false,
      },
    });
  };

  handleEditTextClick = (assetName, templateobj) => {
    const { dispatch } = this.props;
    //相同的另外的key
    let {  asset } = templateobj;
    this.equlist = [];
    this.textList.forEach((v) => {
      if (
        asset.ui &&
        asset.ui.link &&
        asset.ui.link.length &&
        asset.ui.link.indexOf(v.asset.ui.index) !== -1 &&
        assetName !== v.assetName //避免非相同字相同框内字体自适应
      ) {
        this.equlist.push(v.assetName);
      }
    });
    dispatch({
      type: "workBench/save",
      payload: {
        editWindowCurrentIndex: 3,
        currentMaterialKey: assetName,
        isShowTransform: false,
        currentWatermarkKey: "",
        isShowWatermarkTransform: false,
        equalList: this.equlist,
      },
    });
  };

  handleSelectImageClick = (assetName) => {
    const { dispatch } = this.props;
    dispatch({
      type: "workBench/save",
      payload: {
        currentMaterialKey: assetName,
        isShowTransform: true,
        currentWatermarkKey: "",
        isShowWatermarkTransform: false,
      },
    });
  };
  //限制图片/视频大小
  checkFileSize = (fileList) => {
    let flag = true;
    fileList.forEach((v, i) => {
      const fileSize = v.size / 1024 / 1024; //得到的是Mb
      if (isVideo(v.name)) {
        //走到这里说明是视频-----限制视频大小在0-50M
        if (fileSize > 50) {
          flag = false;
        }
      } else {
        //走到这里说明上传的是图片-----限制图片大小在0-12M
        if (fileSize > 12) {
          flag = false;
        }
      }
    });
    return flag;
  };

  handleFileChange = async (e, key) => {
    const { checkFileSize } = this;
    let {
        dispatch,
        assetList,
        editCurrentGroupKey,
        allMaterialByGroup,
        allOriginMaterialByGroup,
      } = this.props,
      file = e.target.files[0],
      fileList = [...e.target.files];
    if (!checkFileSize(fileList)) {
      message.info("仅支持12M以内图片或50M以内视频");
      return;
    }
    let upload = new Upload({
      getTokenUrl: env.getTokenUrl,
      jwt: Cookie.get(TOKEN_NAME),
    });
    let imageUrl = await upload.uploadFile(file);
    let isVideoFile = isVideo(wipeOffQueryString(imageUrl));

    if (isVideoFile) {
      imageUrl += "?x-oss-process=video/snapshot,t_0";
    }

    dispatch({
      type: "workBench/replaceImage",
      payload: {
        assetName: key,
        assetList: assetList,
        editCurrentGroupKey: editCurrentGroupKey,
        allMaterialByGroup: allMaterialByGroup,
        allOriginMaterialByGroup: allOriginMaterialByGroup,
        dataItem: {
          imageSrc: imageUrl,
        },
      },
    });
  };

  renderImage = (material, assetList) => {
    const { assetName } = material,
      imageSrc = assetList[assetName],
      { handleEditImageClick, handleSelectImageClick } = this;
   
    return (
      <div key={assetName} className={`flex ${styles.materialItem}`}>
        <div
          className={`flex cursorPointer ${styles.editTextInfo}`}
          onClick={handleEditImageClick.bind(this, assetName)}
        >
           <img
           className={styles.modify_icon}
          draggable="false"
          src={require('@/assets/img/modify_icon.svg')}
          alt=""
        />
          编辑
        </div>
        {/* <input
          type="file"
          style={this.commonInputStyle}
          id={inputFileId}
          onChange={(e) => handleFileChange(e, assetName)}
        />
        <label
          htmlFor={inputFileId}
          className={`flex cursorPointer ${styles.replaceImageText}`}
        >
          替换图片/视频
        </label> */}
        <img
          draggable="false"
          className={`cursorPointer`}
          onClick={handleSelectImageClick.bind(this, assetName)}
          src={imageSrc}
          alt="assetImg"
        />
      </div>
    );
  };

  renderTextarea = (assetName, defaultText, maxLength, templateobj) => {
    const {
        handleTextBlur,
        handleTextInput,
        handleTextFocus,
        setDomRef,
        setTextRef,
        handleTextClick,
      } = this,
      { textAreaShowStateList } = this.state;
    return (
      <>
        <textarea
          ref={(ref) => setTextRef(ref, assetName)}
          className={`${styles.editTextarea} ${
            !textAreaShowStateList[assetName] && "dnHidden"
          }`}
          maxLength={maxLength}
          value={defaultText}
          onFocus={handleTextFocus.bind(this, assetName)}
          onBlur={(e) => handleTextBlur(assetName, e, maxLength)}
          onInput={(e) => handleTextInput(e, maxLength, assetName, templateobj)}
        />
        <div
          onClick={handleTextClick.bind(this, assetName, templateobj)}
          className={`cursorPointer  ${
            textAreaShowStateList[assetName] ? "dnHidden" : ""
          }`}
          ref={(ref) => setDomRef(ref, assetName)}
        >
          {defaultText}
        </div>
      </>
    );
  };

  renderText = (material) => {
    const { assetName, fontSetting } = material,
      { defaultText, maxLength } = fontSetting,
      { renderTextarea, handleEditTextClick } = this;
    return (
      <div
        key={assetName}
        className={`${styles.materialItem} ${styles.textItem}`}
      >
        <div
          onClick={handleEditTextClick.bind(this, assetName, material)}
          className={`cursorPointer flex  ${styles.editTextInfo}`}
        >
              <img
           className={styles.modify_icon}
          draggable="false"
          src={require('@/assets/img/modify_icon.svg')}
          alt=""
        />
          编辑
        </div>
        {renderTextarea(assetName, defaultText, maxLength, material)}
        <div className={`${styles.textCountContainer}`}>
          {`${defaultText ? defaultText.length : 0}/${maxLength}`}
        </div>
      </div>
    );
  };

  renderList = () => {
    this.textList = [];
    const { assetList, editCurrentGroupKey, allMaterialByGroup } = this.props,
      { renderImage, renderText } = this,
      allMaterial = allMaterialByGroup[editCurrentGroupKey];
    if (!allMaterial) return null;
    return allMaterial.map((material) => {
      let { assetName, fontSetting, asset } = material;
      let uiResult = null;
      switch (material.type) {
        case 1:
          uiResult = renderImage(material, assetList);
          break;

        case 2:
        case 3:
          //取出索引，待点击事件时，循环遍历找出相同的元素，并执行dispatch
          let newobj = {
            assetName,
            asset,
            fontSetting,
          };
          this.textList.push(newobj);
          uiResult = renderText(material);
          break;

        default:
          uiResult = null;
          break;
      }
      return uiResult;
    });
  };

  handleEditWatermarkClick = (id) => {
    const { dispatch } = this.props;
    dispatch({
      
      type: "workBench/save",
      payload: {
        currentMaterialKey: "",
        isShowTransform: false,
        currentWatermarkKey: id,
        editWindowCurrentIndex: 4,
        isShowWatermarkTransform: true,
      },
    });
  };

  handleWatermarkImageClick = (id) => {
    const { dispatch } = this.props;
    dispatch({
      type: "workBench/save",
      payload: {
        currentMaterialKey: "",
        isShowTransform: false,
        currentWatermarkKey: id,
        isShowWatermarkTransform: true,
      },
    });
  };

  renderWatermarkList = () => {
    const { watermarkList } = this.props,
      { handleEditWatermarkClick, handleWatermarkImageClick } = this;

    return watermarkList.map((item) => {
      const { url, id } = item;
      return (
        <div key={id} className={`flex ${styles.materialItem}`}>
          <div
            className={`flex cursorPointer ${styles.editTextInfo}`}
            onClick={handleEditWatermarkClick.bind(this, id)}
          >
            <span
              className={`iconfont ${styles.editIcon}`}
              dangerouslySetInnerHTML={{ __html: "&#xe6a9;" }}
            />
            编辑
          </div>
          <img
            onClick={handleWatermarkImageClick.bind(this, id)}
            className={`cursorPointer`}
            src={url}
            alt="assetImg"
          />
        </div>
      );
    });
  };

  render() {
    const { renderList, renderWatermarkList } = this;

    return (
      <Scrollbars>
        <div className={`flex ${styles.globalContainer}`}>
          <div className={`${styles.materialContainer}`}>
            {renderList()}
            {renderWatermarkList()}
          </div>
        </div>
      </Scrollbars>
    );
  }
  componentDidMount() {}
}

const stateToProps = (state) => {
  const { workBench } = state,
    {
      allMaterialByGroup,
      assetList,
      editCurrentGroupKey,
      currentMaterialKey,
      watermarkList,
      allOriginMaterialByGroup,
    } = workBench;

  return {
    assetList: assetList,
    editCurrentGroupKey: editCurrentGroupKey,
    allMaterialByGroup: allMaterialByGroup,
    currentMaterialKey: currentMaterialKey,
    watermarkList: watermarkList,
    allOriginMaterialByGroup: allOriginMaterialByGroup,
  };
};

export default connect(stateToProps)(EditGlobal);
