import React, { PureComponent } from "react";
import { connect } from "dva";
import Cropper from "utils/lib/Cropper";
import Affine from "utils/lib/Affine";
import ReplaceableJson from "utils/lib/ReplaceableJson";
import { getImage } from "utils";
import Upload from "utils/lib/Upload";
import env from "@/env";
import { Button, Modal, Message } from 'antd'
import styles from "./index.less";
import Cookies from "js-cookie";
import { TOKEN_NAME, PAGE_LOADING_TYPE } from "@/types";
import { openNotification } from "utils/userPrompt";
import app from "../../utils/lib/app";
const { confirm } = Modal;
class RenderButtonComponent extends PureComponent {

  constructor(props) {
    super(props);
    this.state = {
      isModalVisible: false,
      loading: true,
      costTime: 0,
      videoUrl: '',
      isBack: true,
      muted: false,
    };
    this.timerId = null;
    this.scrollStyle = {
      flex: "1",
    };

  }



  mapGroups(group) {
    let upload = new Upload({
      getTokenUrl: env.getTokenUrl,
      jwt: Cookies.get(TOKEN_NAME),
      sourceHost: env.sourceHost,
    });


    const mapCb = async material => {

      let {
        type, assetName, originSize, isVideo,
        key, volume, videoClip,
      } = material;
      const { assetList, parseManage } = this.props["workBench"],
        currentUi = parseManage["uiAssets"][key]["ui"],
        materialWidth = currentUi["editSize"][0],
        materialHeight = currentUi["editSize"][1],
        originWidth = originSize[0],
        originHeight = originSize[1],
        cropOptions = {
          width: materialWidth,
          height: materialHeight,
        },
        result = {
          key,
          type,
        };


      if (type === 1) {
        let url = assetList[assetName],
          matrix = {
            anchor: {
              x: material.anchor[0],
              y: material.anchor[1],
            },
            position: {
              x: material.position[0],
              y: material.position[1],
            },
            zoom: {
              x: material.scale[0],
              y: material.scale[1],
            },
            rotation: material.rotate,
          };

        if (isVideo) {
          result["imgUrl"] = url.substring(0, url.lastIndexOf("?"));
          result["isVideo"] = isVideo;
          result["adapt_type"] = 4;
          result["volume"] = volume;
          result["videoClip"] = videoClip;
          let videoMatrix = new Affine();
          videoMatrix.set(matrix.anchor, matrix.position, matrix.zoom, matrix.rotation);

          result["videoTransform"] = [
            videoMatrix.mA, videoMatrix.mC, videoMatrix.mE,
            videoMatrix.mB, videoMatrix.mD, videoMatrix.mF,
          ];

          return result;
        }

        let crop = new Cropper(cropOptions),
          img = await getImage(url);
        crop.setAffine(matrix);
        // console.log(`${assetName} 开始截图...`);
        await crop.drawImage(img, 0, 0, originWidth, originHeight);
        // console.log(`${assetName} 截图完成!`);
        let bolbData = await crop.getBlobData(),
          fileData = new File([bolbData], assetName, { type: bolbData.type });
        // console.log(`${assetName} 开始上传...`);
        result["imgUrl"] = await upload.uploadFile(fileData);
        // console.log(`${assetName} 上传完成`);
        // console.log(result["imgUrl"]);
      }

      if (type === 2 || type === 3) {
        result["fontSetting"] = material["fontSetting"];
      }

      return result;
    };


    return Promise.all(group.map(mapCb));
  }

  async mapAllMaterialByGroup(allMaterialByGroup) {
    let key,
      currentGroup,
      resultJson = {};
    for (key in allMaterialByGroup) {
      if (allMaterialByGroup.hasOwnProperty(key)) {
        currentGroup = allMaterialByGroup[key];
        resultJson[key] = await this.mapGroups(currentGroup);
      }
    }

    return resultJson;
  };

  dispatchMiddle = (type, payload) => {
    const { dispatch } = this.props;
    dispatch({
      type: `editor/${type}`,
      payload: payload,
    });
  };
  veLeapTestCheck = () => {
    const { isVeLeapTest } = this.props["editor"];
    let count = 0;
    if (!isVeLeapTest) return count;
    const { allMaterialByGroup, allOriginMaterialByGroup, assetList, originAssetList } = this.props['workBench'];
    for (var i in allMaterialByGroup) {
      allMaterialByGroup[i].forEach((item, index) => {
        if (item.type === 1) {
          if (assetList[item.assetName] === originAssetList[item.assetName]) {
            count++
          }
        }  else{
          if (item.fontSetting.defaultText === allOriginMaterialByGroup[i][index].fontSetting.defaultText) {
            count++
          }

        }
      })
    }
    return count
  }
  
  renderText = () => {
    const { loading, costTime } = this.state;
    const { duration_sec } = this.props;
    const flag = costTime / 1000 < duration_sec * 0.8;
    return (
      !loading ?
        <div className={styles.renderText}>
          <div className={styles.renderingTime}>渲染时长：{(costTime / 1000).toFixed(1)}s</div>
          <div className={`${styles.renderSpeed} ${!flag ? styles.redSpeed : ''}`}>
            {flag ? '渲染速度正常' : '渲染速度较慢'}
          </div>
        </div>
        : ''
    )
  };
  handleCancel = () => {
    this.setState({
      isModalVisible: false,
      muted: true,
    })
  };

  renderModelContent = () => {
    const { loading, videoUrl, muted } = this.state;
    return (

      loading ?
        // <div className={styles.content}>
        //   <div className={styles.loadingBox}>
        //     <div className={`flex ${styles.loaderContainer}`}>
        //       <div className={`${styles.eight}`} />
        //     </div>
        //   </div>
        //   <p>渲染中，请稍等...</p>
        // </div>
        <div className={styles.watiImageBox}>
          <img  src="http://static.atvideo.cc/2021/09/10/15/55/20210910-155302.gif" alt="" />
        </div>
        : <video controls autoPlay muted={muted} className={styles.video} src={'http:' + videoUrl}></video>


    )
  };
  submitBtn = () => {
    // const { isBack } = this.state;
    // if (!isBack) {
    //   this.setState({
    //     isModalVisible: false
    //   })
    //   return
    // }
    if(!app.getUrlValue('open')){
    confirm({
      content: <div>是否返回上一级</div>,
      cancelText: <span>取消</span>,
      okText: <span>确定</span>,
      onOk() {
        // window.open('http://veleap.com/web/center/works?action=test&page=1&page_size=6')
        window.history.back(-1); 
      },
      onCancel() {
      },
    })
  }else{
    this.setState({
      isModalVisible:false,
      muted: true
    })
  }

  };
  //把视频和时长发给后端
  sendRenderResult = () => {
    let demoToken = app.getUrlValue('v');
    app.request(env.veLeapHost + '/web/work/render_result', {
      edit_video_url: this.state.videoUrl,
      edit_render_time: this.state.costTime,
      id: app.getUrlValue('test_id'),
      token: demoToken
    }).then(res => {
    })
  };
  //是否渲染
  getWhetherRender = () => {
    let demoToken = app.getUrlValue('v');
    app.request(env.veLeapHost + '/leap/designer/get_render_url', {
      access_token: app.getUrlValue('a'),
      test_id: app.getUrlValue('test_id'),
      token: demoToken
    }).then(res => {
      if (res.errno !== 0) {
        Message.error(res.errormsg)
        return
      }
      //若存在视频地址在赋值
      if(res.data.edit_video_url){
        this.setState({
          videoUrl: res.data.edit_video_url,
          costTime: res.data.edit_render_time,
          isModalVisible: true,
          loading: false,
          isBack: false,
          muted: false,
        })
      }
    })
  }
  //点击按钮，提交渲染
  submitRendering = () => {

    const { pageLoading, currentTemplateInfo, isVeLeapTest } = this.props["editor"],
      { a, pid, v, i, test_adopt, project_id, process_id } = this.props.query,
      { allMaterialByGroup, parseManage } = this.props['workBench'];
    const { dispatch } = this.props;
    let noReplaceCount = this.veLeapTestCheck()
    if (noReplaceCount) {
      openNotification("不符合测试要求!", `还剩${noReplaceCount}个素材未替换`, "error", 10);
      return;
    }


    if (pageLoading) return;
    dispatch({
      type: `editor/save`,
      payload: {
        pageLoading: true,
        pageLoadingText: "正在上传文件",
        isShowCurrentTransform: false,
        currentMaterialIndex: -1,
        editWindowCurrentIndex: 1,
      },
    });
    

    localStorage.setItem(PAGE_LOADING_TYPE, "1");
    this.mapAllMaterialByGroup(allMaterialByGroup).then(res => {
      dispatch({
        type: `editor/save`,
        payload: {
          pageLoading: false,
        },
      });
      localStorage.setItem(PAGE_LOADING_TYPE, "0");
      const replaceJson = new ReplaceableJson(res, parseManage.orderList, currentTemplateInfo.id),
        json = replaceJson.getJson(),
        renderType = isVeLeapTest ? "veLeapTestSubmitRender" : "submitRender";
      //说明是模板页面过来的
      const payload = {
        json: json,
        veLeapToken: v,
        a,
        pid,
        i,
        test_adopt,
        project_id,
        process_id,
      }
      if (app.getUrlValue('test_id')) {
        const { json } = payload;

        this.setState({
          isModalVisible: true,
          loading:true,
          videoUrl:'',
          muted: false,
        })

       
          app.request('/demo/main/start_render', json).then(response => {
            const { data } = response;
            this.timerId = setInterval(() => {
              app.request('/demo/public/get_status', { order_id: data.res.data["orderId"] }).then(data => {
                if (data.errno !== 0) {
                  clearInterval(this.timerId);
                }
                if (data.data.order.status === 1) {
                  this.setState({
                    loading: false
                  })
                  this.setState({
                    costTime:  data.data.cost_time,
                    videoUrl: env.demoH5Host + data.data.order.video_url,
                    muted: false,
                  }, () => {
                    this.sendRenderResult()
                  })
                  clearInterval(this.timerId);
                }
              })
            }, 500);
          })
        return
      }



      dispatch({
        type: `editor/${renderType}`,
        payload,
      });

    });


  };
  componentDidMount() {
    //是交付系统过来的才让弹框
    // if(app.getUrlValue('test_id')){
    //   this.getWhetherRender()
    // }
  };
  render() {

    const {
      isOpenMaterialList, groups, editorContainerHeight, totalTime,
    } = this.props;
    const { isModalVisible, loading } = this.state;

    return (
      <div>
        <div onClick={this.submitRendering} style={{
          zIndex: 500,
          position: 'absolute',
          right: '300px',
          top: '80px',

        }}>
          <Button type="primary">提交渲染</Button>

        </div>
        <Modal
          width='668px'
          footer={null}
          title="渲染结果"
          centered={true}
          visible={isModalVisible}
          onCancel={this.handleCancel}
        >
          <div className={styles.content_page}>
            {this.renderModelContent()}
            {this.renderText()}
            <div  className={styles.submitBtn} onClick={this.submitBtn}>
              确定
            </div>
          </div>
        </Modal>

      </div>




    );
  }

}

const stateToProps = (state) => {
  let loading = state["loading"]["global"];
  return {
    loadingEffects: loading,
    query: state["router"]["location"]["query"],
    editor: state["editor"],
    workBench: state['workBench'],
    duration_sec: state['workBench'].duration_sec
  };
};

export default connect(stateToProps)(RenderButtonComponent);

