import React, { useEffect, useState } from "react";
import { connect } from "dva";
import styles from "./index.less";
import axios from "axios";
import env from "@/env";

function VideoPlay(props) {
  const [loading, setLoading] = useState(true),
    [url, setUrl] = useState(""),
    [constTime, setConstTime] = useState(0),

    { orderId, video_url } = props.location.query;

  
  useEffect(() => {
    let timerId = -1;
    const pollVideoIsLoadSuccess = (id) => {
      return new Promise(resolve => {
        timerId = setInterval(async () => {
          const { data } = await axios({
            url: `${env.host}/TemplateDemo.php?act=get_status&record_id=${orderId}`,
            method: "GET",
            data: {
              order_id: id,
            },
          });
          
          if (data.errno !== 0) {
            clearInterval(timerId);
          }

          if (data.data.status === "1") {
            clearInterval(timerId);
            setConstTime(data.data.cost_time/1000)
            resolve(`${env.sourceHost}${video_url}`);
          }


        }, 500);
      });

    };


    pollVideoIsLoadSuccess(orderId).then((url) => {
      setLoading(false);
      setUrl(url);
    });

    return () => {
      clearInterval(timerId);
    };

  }, [orderId, video_url]);

 const renderText=()=>{
   const {duration_sec}=props;
   const flag = constTime<duration_sec*0.8;
    return(
      <div className={styles.renderText}>
        <div className={styles.renderingTime}>渲染时长：{constTime.toFixed(1)}s</div>
        <div  className={`${styles.renderSpeed} ${!flag?styles.redSpeed:''}`}>
           {flag?'渲染速度正常':'渲染速度较慢'}
        </div>
      </div>
    )
  }
  
  return loading ? (
  <div className={styles.content}>
      <div className={styles.loadingBox}>
        <div className={`flex ${styles.loaderContainer}`}>
          <div className={`${styles.eight}`}/>
        </div>
      </div>
      <p>正在处理中...</p>
    </div>
  
  ) : (
    <div>

    <div className={styles.videoPlay}>
      <video
        src={url}
        style={{ width: "1109px", height: "625px", outline: "none" }}
        controls
      />
    </div>
    {renderText()}
    </div>

  );
}

const stateToProps = state => {
  return {
    editor: state["editor"],
    loading: state["loading"],
    duration_sec:state['workBench'].duration_sec
  };
};

export default connect(stateToProps)(VideoPlay);

