export function getEditContainerRectInfo(
  w,
  h,
  imageWidth,
  imageHeight,
  levelSpace = 0,
  verticalSpace = 0,
) {
  const windowHeight = window.innerHeight,
    windowWidth = window.innerWidth,
    expectHeight = windowHeight - verticalSpace,
    expectWidth = windowWidth - levelSpace;

  let scale = expectHeight / h,
    imgScale = expectHeight / imageHeight,
    imgWidth = imageWidth * imgScale,
    imgHeight = expectHeight,
    width = w * scale,
    height = expectHeight,
    result = {
      width: width,
      height: height,
      scale: scale,
      imgWidth: imgWidth,
      imgHeight: imgHeight,
      imgScale: imgScale,
    };

  if (expectWidth <= width) {
    imgScale = expectWidth / imageWidth;
    imgHeight = imageHeight * imgScale;
    imgWidth = expectWidth;
    scale = expectWidth / w;
    width = expectWidth;
    height = w * scale;
    result = {
      width: width,
      height: height,
      scale: scale,
      imgWidth: imgWidth,
      imgHeight: imgHeight,
      imgScale: imgScale,
    };
  }

  return result;
}

export function getEditInfo(p, imgWidth, imgHeight) {
  let proportion = p,
    imageWidth = imgWidth,
    imageHeight = imgHeight,
    w = 0,
    h = 0,
    a = 9 / 16,
    b = 0.65625,
    c = 3 / 4,
    d = 0.875,
    e = 1 / 1,
    f = (1 / 1 + 16 / 9) / 2;

  // 画布尺寸 9:16
  let cA = 540,
    cB = 630,
    cC = 900;

  // 画布尺寸 1:1
  let oA = 720,
    oB = 760,
    oC = 940;

  // 画布尺寸 16:9
  let nA = 1280,
    nB = 1440,
    nC = 1760;

  if (proportion <= a || (a < proportion && proportion <= b)) {
    if (imageWidth <= cA) {
      w = 540;
      h = 960;
    } else if (cA < imageWidth && imageWidth <= cB) {
      w = 540;
      h = 960;
    } else if (cB < imageWidth && imageWidth <= 720) {
      w = 720;
      h = 1280;
    } else if (720 < imageWidth && imageWidth <= cC) {
      w = 720;
      h = 1280;
    } else if (cC < imageWidth) {
      w = 1080;
      h = 1920;
    }
  } else if (
    (proportion > b && proportion <= c) ||
    (proportion > c && proportion <= d)
  ) {
    w = 750;
    h = 1000;
  } else if (
    (proportion > d && proportion <= e) ||
    (proportion > e && proportion <= f)
  ) {
    if (imageWidth <= oA) {
      w = 720;
      h = 720;
    } else if (oA < imageWidth && imageWidth <= oB) {
      w = 720;
      h = 720;
    } else if (oB < imageWidth && imageWidth <= 800) {
      w = 800;
      h = 800;
    } else if (800 < imageWidth && imageWidth <= oC) {
      w = 800;
      h = 800;
    } else if (oC < imageWidth) {
      w = 1080;
      h = 1080;
    }
  } else if (proportion > f) {
    if (imageWidth <= nA) {
      w = 1280;
      h = 720;
    } else if (nA < imageWidth && imageWidth <= nB) {
      w = 1280;
      h = 720;
    } else if (nB < imageWidth && imageWidth <= 1600) {
      w = 1600;
      h = 900;
    } else if (1600 < imageWidth && imageWidth <= nC) {
      w = 1600;
      h = 900;
    } else if (oC < imageWidth) {
      w = 1920;
      h = 1080;
    }
  }
  return { w, h, imageWidth, imageHeight };
}

export function getEditProportionInfo(p, imgWidth, imgHeight, index) {
  let proportion = p,
    imageWidth = imgWidth,
    imageHeight = imgHeight,
    w = 0,
    h = 0,
    a = 9 / 16,
    b = 0.65625,
    c = 3 / 4,
    d = 0.875,
    e = 1 / 1,
    f = (1 / 1 + 16 / 9) / 2;

  // 画布尺寸 9:16
  let cA = 540,
    cB = 630,
    cC = 900;

  // 画布尺寸 1:1
  let oA = 720,
    oB = 760,
    oC = 940;

  // 画布尺寸 16:9
  let nA = 1280,
    nB = 1440,
    nC = 1760;

  switch (Number(index)) {
    case 0:
      if (proportion <= a || (a < proportion && proportion <= b)) {
        if (imageWidth <= cA) {
          w = 540;
          h = 960;
        } else if (cA < imageWidth && imageWidth <= cB) {
          w = 540;
          h = 960;
        } else if (cB < imageWidth && imageWidth <= 720) {
          w = 720;
          h = 1280;
        } else if (720 < imageWidth && imageWidth <= cC) {
          w = 720;
          h = 1280;
        } else if (cC < imageWidth) {
          w = 1080;
          h = 1920;
        }
      } else if (
        (proportion > b && proportion <= c) ||
        (proportion > c && proportion <= d)
      ) {
        w = 750;
        h = 1000;
      } else if (
        (proportion > d && proportion <= e) ||
        (proportion > e && proportion <= f)
      ) {
        if (imageWidth <= oA) {
          w = 720;
          h = 720;
        } else if (oA < imageWidth && imageWidth <= oB) {
          w = 720;
          h = 720;
        } else if (oB < imageWidth && imageWidth <= 800) {
          w = 800;
          h = 800;
        } else if (800 < imageWidth && imageWidth <= oC) {
          w = 800;
          h = 800;
        } else if (oC < imageWidth) {
          w = 1080;
          h = 1080;
        }
      } else if (proportion > f) {
        if (imageWidth <= nA) {
          w = 1280;
          h = 720;
        } else if (nA < imageWidth && imageWidth <= nB) {
          w = 1280;
          h = 720;
        } else if (nB < imageWidth && imageWidth <= 1600) {
          w = 1600;
          h = 900;
        } else if (1600 < imageWidth && imageWidth <= nC) {
          w = 1600;
          h = 900;
        } else if (oC < imageWidth) {
          w = 1920;
          h = 1080;
        }
      }
      return { w, h, imageWidth, imageHeight };
      break;
    case 1:
      if (imageWidth <= nA) {
        w = 1280;
        h = 720;
      } else if (nA < imageWidth && imageWidth <= nB) {
        w = 1280;
        h = 720;
      } else if (nB < imageWidth && imageWidth <= 1600) {
        w = 1600;
        h = 900;
      } else if (1600 < imageWidth && imageWidth <= nC) {
        w = 1600;
        h = 900;
      } else if (oC < imageWidth) {
        w = 1920;
        h = 1080;
      }

      return { w, h, imageWidth, imageHeight };

    case 2:
      if (imageWidth <= cA) {
        w = 540;
        h = 960;
      } else if (cA < imageWidth && imageWidth <= cB) {
        w = 540;
        h = 960;
      } else if (cB < imageWidth && imageWidth <= 720) {
        w = 720;
        h = 1280;
      } else if (720 < imageWidth && imageWidth <= cC) {
        w = 720;
        h = 1280;
      } else if (cC < imageWidth) {
        w = 1080;
        h = 1920;
      }
      return { w, h, imageWidth, imageHeight };

    case 3:
      if (imageWidth <= oA) {
        w = 720;
        h = 720;
      } else if (oA < imageWidth && imageWidth <= oB) {
        w = 720;
        h = 720;
      } else if (oB < imageWidth && imageWidth <= 800) {
        w = 800;
        h = 800;
      } else if (800 < imageWidth && imageWidth <= oC) {
        w = 800;
        h = 800;
      } else if (oC < imageWidth) {
        w = 1080;
        h = 1080;
      }
      return { w, h, imageWidth, imageHeight };

    case 4:
      w = 750;
      h = 1000;
      return { w, h, imageWidth, imageHeight };

    default:
      break;
  }
}

export function getPosititon(sW, sH, stickers) {
  let stickersList = [...stickers];
  for (let i = 0; i < stickersList.length; i++) {
    let currentSticker = stickersList[i]["stickersStyle"];

    currentSticker["x"] = currentSticker["x"] / sW;
    currentSticker["y"] = currentSticker["y"] / sH;
    if (sW >= sH) {
      currentSticker["sX"] = currentSticker["sX"] * sH;
      currentSticker["sY"] = currentSticker["sY"] * sH;
    } else {
      currentSticker["sX"] = currentSticker["sX"] * sW;
      currentSticker["sY"] = currentSticker["sY"] * sW;
    }

    stickersList[i]["stickersStyle"] = { ...currentSticker };
  }
  return { stickersList };
}

export function getWaterPosititon(sW, sH, watermarkListStickers) {
  let watermarksList = [...watermarkListStickers];
  for (let i = 0; i < watermarksList.length; i++) {
    let currentSticker = watermarksList[i];
    currentSticker["position"] = [
      currentSticker["position"][0] / sW,
      currentSticker["position"][1] / sH,
    ];

    if (sW >= sH) {
      currentSticker["scale"] = [
        currentSticker["scale"][0] / sH,
        currentSticker["scale"][1] / sH,
      ];
    } else {
      currentSticker["scale"] = [
        currentSticker["scale"][0] / sW,
        currentSticker["scale"][1] / sW,
      ];
    }

    watermarksList[i] = { ...currentSticker };
  }
  return { watermarksList };
}
